package auxiliary_folder_handling;

import gui.active_learning_tool.*;
import instance_provider.Instance_provider;
import instance_provider.Train_file_instance_provider;
import java.io.*;
import java.util.*;
import classifier.*;
import javax.swing.*;
import document.*;

public class Proposer
{

	public Proposer(String path)
	{
		file_separator = System.getProperty("file.separator");
		file_separator = (file_separator.equals("\\"))?"\\\\":file_separator;
		auxiliary_path = path;
		window = -1;
		window_examples = null;
		best_examples = null;
		//the parameters
		entropy_plus_diversity = false;
		basic_tagset = false;
		gain_ratio = false;
		k = 1;
		MVDM = false;
		BufferedReader reader = null;
		try
		{
			reader = new BufferedReader(new InputStreamReader(
					new FileInputStream(new File(auxiliary_path + "" + file_separator + "parameters.txt")), "UTF-8"));
			entropy_plus_diversity = (reader.readLine().equals("true")?true:false);
			basic_tagset = (reader.readLine().equals("true")?true:false);
			gain_ratio = (reader.readLine().equals("true")?true:false);
			k = Integer.parseInt(reader.readLine());
			MVDM = (reader.readLine().equals("true")?true:false);
		}
		catch(Exception e)
		{
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, "Problem occured while reading the parameter file.");
		}
		finally
		{
			try{reader.close();}catch(Exception e){e.printStackTrace();}
		}
		new Pool_parser().execute();
	}
	
	//invoked when the parsing of the pool file is completed
	public void parsing_completed()
	{
		if(window == -1)
		{
			window = (all_examples-train_examples)/10;
			examples_to_propose = window/10;
		}
		frame = new Active_learning_tool_proposal_window(this,
				all_examples-train_examples, train_examples, window, examples_to_propose,
				entropy_plus_diversity, basic_tagset, gain_ratio, k, MVDM);
		frame.init();
	}
	
	//invoked when proposal is requested
	public void request_to_propose(int window, int new_examples)
	{
		if(window_examples == null)
		{
			new Window_retriever(new_examples).execute();
			return;
		}
		Selector selector = new Selector(get_classifier(),window_examples, new_examples,
				best_examples, this, window);
		//get the most useful examples
		if(best_examples == null)
		{
				selector.execute();
				return;
		}
		//find the sentences where these words appear
		Object[] sentences_and_highlight = get_sentences_and_highlights_for_examples(best_examples);

		//see whether we need the extended tagset or the basic one
		boolean extended_tagset = false;
		BufferedReader reader = null;
		try
		{
			reader = new BufferedReader(new InputStreamReader(
					new FileInputStream(new File(auxiliary_path + "" + file_separator + "parameters.txt")), "UTF-8"));
			//skip the first line
			reader.readLine();
			extended_tagset = ((String)reader.readLine()).equals("false")?true:false;
		}
		catch(Exception e)
		{
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, "Problem occured while reading the parameter file.");
		}
		finally
		{
			try{reader.close();}catch(Exception e){e.printStackTrace();}
		}

		//show the sentences
		frame.dispose();
		String[] words = new String[best_examples.length];
		for(int i=0; i<best_examples.length; ++i)
		{
			words[i] = best_examples[i].split(" ")[4];
		}
		//nullify the window
		window_examples = null;

		//get the knn's opinion for the categories
		categories = new String[best_examples.length];
		Knn knn = get_classifier();
		for(int i=0; i<best_examples.length; ++i)
		{
			if(knn != null)
			{
				categories[i] = knn.classify(selector.get_instance(best_examples[i], knn));
			}
			else
			{
				categories[i] = "--/--/--/--/--";
			}
		}
		
		new Active_learning_tool_category_filling(this, extended_tagset,
				(String[])sentences_and_highlight[0],
				(int[])sentences_and_highlight[1], (int[])sentences_and_highlight[2], words,
				categories).init();
	}

	//saves the train file in the path location
	public void request_to_save_train_file(File path)
	{
		BufferedReader reader = null;
		PrintWriter writer = null;
		try
		{
			reader = new BufferedReader(new InputStreamReader(new FileInputStream(
					new File(auxiliary_path + "" + file_separator + "train.train")), "UTF-8"));
			String line = reader.readLine();
			ArrayList<String> list = new ArrayList<String>();
			while(line != null)
			{
				list.add(line);
				line = reader.readLine();
			}
			writer = new PrintWriter(path.getAbsolutePath() + ".train", "UTF-8");
			Iterator<String> it = list.iterator();
			while(it.hasNext())
			{
				String temp = it.next();
				if(temp == list.get(list.size()-1))
				{
					writer.print(temp);
				}
				else
				{
					writer.println(temp);
				}
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, "Problem occured while exporting the train file.");
		}
		finally
		{
			try{reader.close();writer.close();}catch(Exception e){e.printStackTrace();}
		}
	}
	
	//invoked when the categories have been set
	public void categories_are_set(String[] categories)
	{
		PrintWriter writer = null;
		BufferedReader reader = null;
		//save the new train examples
		try
		{
			reader = new BufferedReader(new InputStreamReader(
					new FileInputStream(auxiliary_path + "" + file_separator + "train.train"), "UTF-8"));
			String line = reader.readLine();
			ArrayList<String> list = new ArrayList<String>();
			while(line != null)
			{
				list.add(line);
				line = reader.readLine();
			}
			reader.close();
			writer = new PrintWriter(auxiliary_path + "" + file_separator + "train.train", "UTF-8");
			Iterator<String> it = list.iterator();
			while(it.hasNext())
			{
				writer.println(it.next());
			}
			for(int i=0; i<categories.length-1; ++i)
			{
				writer.print(best_examples[i].split(" ")[2] + " ");
				writer.print(best_examples[i].split(" ")[3] + " ");
				writer.print(best_examples[i].split(" ")[4] + " ");
				writer.print(categories[i] + " ");
				writer.println(best_examples[i].split(" ")[5]);
			}
			writer.print(best_examples[categories.length-1].split(" ")[2] + " ");
			writer.print(best_examples[categories.length-1].split(" ")[3] + " ");
			writer.print(best_examples[categories.length-1].split(" ")[4] + " ");
			writer.print(categories[categories.length-1] + " ");
			writer.print(best_examples[categories.length-1].split(" ")[5]);
		}
		catch(Exception e)
		{
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, "Problem occured while writing the train file.");
		}
		finally
		{
			try{writer.close();}catch(Exception e){e.printStackTrace();}
		}
		
		new Pool_updater(this).execute();
		
	}
	
	//updates the spinner values
	public void set_spinner_values(int a, int b)
	{
		window = a;
		examples_to_propose = b;
	}
	
	public void save_parameters(boolean entropy_plus_diversity, int k, boolean gain_ratio, boolean MVDM)
	{
		this.entropy_plus_diversity = entropy_plus_diversity;
		this.k = k;
		this.gain_ratio = gain_ratio;
		this.MVDM = MVDM;
		PrintWriter writer = null;
		try
		{
			writer = new PrintWriter(auxiliary_path + "" + file_separator + "parameters.txt", "UTF-8");
			writer.println(entropy_plus_diversity);
			writer.println(basic_tagset);
			writer.println(gain_ratio);
			writer.println(k+"");
			writer.print(MVDM);
		}
		catch(Exception e)
		{
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, "Problem occured while writing the parameters file.");
		}
		finally
		{
			try{writer.close();}catch(Exception e){e.printStackTrace();}
		}
	}
	
	//returns the sentences of the given examples and the highlight markers
	private Object[] get_sentences_and_highlights_for_examples(String[] examples)
	{
		String[] sentences = new String[examples.length];
		int[] start_of_highlight = new int[examples.length];
		int[] end_of_highlight = new int[examples.length];
		ObjectInputStream reader = null;
		try
		{
			//for each example
			for(int i=0; i<sentences.length; ++i)
			{
				//open the corresponding document file
				reader = new ObjectInputStream(
						new FileInputStream(auxiliary_path+"" + file_separator + ""+examples[i].split(" ")[6]));
				Token_document document = (Token_document)reader.readObject();
				reader.close();
				//get the token
				Token token = document.get_token_in_cursor_position(new Integer(examples[i].split(" ")[7]));
				//and its sentence
				Token_sentence sentence = document.get_sentence_of_token(token);
				String temp = "";
				Token current = sentence.get_first_token();
				Token last = sentence.get_last_token();
				int start = 0;
				while(current != last)
				{
					if(current == token)
					{
						start_of_highlight[i] = start;
						end_of_highlight[i] = start + token.get_token().length();
					}
					temp += current.get_token() + " ";
					start += current.get_token().length()+1;
					current = document.get_next_token(current);
				}
				//and the last token
				if(current == token)
				{
					start_of_highlight[i] = start;
					end_of_highlight[i] = start + token.get_token().length();
				}
				temp += current.get_token();
				sentences[i] = temp;
			}
		}
		catch(Exception e)
		{
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, "Problem occured while reading a document file.");
		}
		finally
		{
			try{reader.close();}catch(Exception e){e.printStackTrace();}
		}
		Object[] arrays = new Object[3];
		arrays[0] = sentences;
		arrays[1] = start_of_highlight;
		arrays[2] = end_of_highlight;
		return arrays;
	}
	
	//returns a knn classifier according to the parameters saved in the auxiliary folder
	private Knn get_classifier()
	{
		ArrayList<Instance> train_set = get_train_instances();
		if(train_set.size() == 0)
		{
			return null;
		}
		BufferedReader reader = null;
		Knn knn = null;
		try
		{
			reader = new BufferedReader(new InputStreamReader(
					new FileInputStream(new File(auxiliary_path+ "" + file_separator + "parameters.txt")), "UTF-8"));
			//skip first line
			boolean entropy_plus_diversity = (reader.readLine().equals("true"))?true:false;
			//skip second line
			reader.readLine();
			//the third line is the gain ratio choice
			boolean gain_ratio = (reader.readLine().equals("true"))?true:false;
			//the forth line is the k choice
			int k = new Integer(reader.readLine());
			//the fifth line is the difference metric choice
			int d_m = (reader.readLine().equals("true"))?0:1;
			//create the classifier
			knn = new Knn(k);
			knn.train(train_set, d_m);
			knn.use_properties_weight(gain_ratio);
			knn.set_selection_measure(entropy_plus_diversity);
		}
		catch(Exception e)
		{
			e.printStackTrace();
			JOptionPane.showMessageDialog(null, "Problem occured while reading the parameters file.");
		}
		finally
		{
			try{reader.close();}catch(Exception e){e.printStackTrace();}
		}
		//return the classifier
		return knn;
	}
	
	//extracts all the train instances
	private ArrayList<Instance> get_train_instances()
	{
		Instance_provider provider = new Train_file_instance_provider();
		provider.set_number_of_instances_to_provide(train_examples);
		provider.parse_file(auxiliary_path + "" + file_separator + "train.train");
		return provider.get_instances();
	}
	
	//this class is used to parse the pool file and count the available train examples
	public class Pool_parser extends SwingWorker<Void, Void>
	{
		
		public Void doInBackground()
		{
			Active_learning_tool_loading_bar bar = new Active_learning_tool_loading_bar("Scanning Examples");
			bar.init();
			bar.progress.setString("0");
			all_examples = 0;
			train_examples = 0;
			BufferedReader reader = null;
			try
			{
				reader = new BufferedReader(new InputStreamReader(
						new FileInputStream(new File(auxiliary_path + "" + file_separator + "pool.txt")), "UTF-8"));
				String line = reader.readLine();
				while(line != null)
				{
					all_examples++;
					bar.progress.setString("Words found: " + all_examples);
					if(line.split(" ")[0].equalsIgnoreCase("true"))
					{
						train_examples++;
					}
					line = reader.readLine();
				}
			}
			catch(Exception e)
			{
				e.printStackTrace();
				JOptionPane.showMessageDialog(null, "Error occured during the auxiliary folder processing.");
			}
			finally
			{
				try{reader.close();}catch(Exception e){e.printStackTrace();}
			}
			bar.setVisible(false);
			return null;
		}
		
		public void done()
		{
			parsing_completed();
		}
		
	}
	
	//returns 'window' examples that have never been used before
	public class Window_retriever extends SwingWorker<Void, Void>
	{
		
		public Window_retriever(int n)
		{
			new_examples = n;
		}
		
		public Void doInBackground()
		{
			//find the line of the first example
			int check_point_line = 0;
			BufferedReader reader = null;
			try
			{
				//get the offset of the first example we should take in the pool file
				reader = new BufferedReader(new InputStreamReader(
						new FileInputStream(new File(auxiliary_path + "" + file_separator + "current_line.txt")), "UTF-8"));
				check_point_line = Integer.parseInt(reader.readLine());
			}
			catch(Exception e)
			{
				e.printStackTrace();
				JOptionPane.showMessageDialog(null, "Problem occured while reading the pool file.");
			}
			finally
			{
				try{reader.close();}catch(Exception e){e.printStackTrace();}
			}

			Active_learning_tool_loading_bar bar = new Active_learning_tool_loading_bar("Retrieving Window");
			bar.init();
			bar.progress.setMaximum(check_point_line + window);
			int counter = 0;
			bar.progress.setValue(counter);
			
			//create the window
			int current_line = 1;
			window_examples = new String[window];
			try
			{
				reader = new BufferedReader(new InputStreamReader(
						new FileInputStream(new File(auxiliary_path + "" + file_separator + "pool.txt")), "UTF-8"));
				String line = reader.readLine();
				//go to the checkpoint line
				while(current_line != check_point_line)
				{
					line = reader.readLine();
					current_line++;
					counter++;
					bar.progress.setValue(counter);
				}
				
				//get the examples from then on
				for(int i=0; i<window;)
				{
					//find the next example
					//if we reached the end of the pool file we start reading from the beginning
					if(line == null)
					{
						reader = new BufferedReader(new InputStreamReader(
								new FileInputStream(new File(auxiliary_path + "" + file_separator + "pool.txt")), "UTF-8"));
						line = reader.readLine();
						current_line = 1;
						if(line == null)
						{
							throw new IllegalArgumentException("Proposer: Empty pool file.");
						}
					}
					//if this line has been used get the next one
					if(line.split(" ")[0].equals("true"))
					{
						line = reader.readLine();
						current_line++;
						continue;
					}
					//else we have found the next example
					else
					{
						//add it to the window
						window_examples[i] = line;
						i++;
						line = reader.readLine();
						current_line++;
						counter++;
						bar.progress.setValue(counter);
					}
				}
			}
			catch(Exception e)
			{
				e.printStackTrace();
				JOptionPane.showMessageDialog(null, "Problem occured while reading the pool file.");
			}
			finally
			{
				try{reader.close();}catch(Exception e){e.printStackTrace();}
			}
			
			//update the current_line file
			PrintWriter writer = null;
			try
			{
				writer = new PrintWriter(auxiliary_path + "" + file_separator + "current_line.txt", "UTF-8");
				writer.println(current_line);
				writer.flush();
			}
			catch(Exception e)
			{
				e.printStackTrace();
				JOptionPane.showMessageDialog(null, "Problem occured while writing the current_line file.");
			}
			finally
			{
				try{reader.close();}catch(Exception e){e.printStackTrace();}
			}
			bar.dispose();
			return null;
		}
		
		public void done()
		{
			request_to_propose(window, new_examples);
		}
		
		private int new_examples;
		
	}
	
	//updates the pool file
	public class Pool_updater extends SwingWorker<Void, Void>
	{
		
		public Pool_updater(Proposer p)
		{
			proposer = p;
		}
		
		public Void doInBackground()
		{
			//show the loading bar
			Active_learning_tool_loading_bar bar =
				new Active_learning_tool_loading_bar("Updating Auxiliary Folder");
			bar.init();
			bar.progress.setMaximum(all_examples*2);
			int counter = 0;
			bar.progress.setValue(counter);
			
			PrintWriter writer = null;
			
			//update the pool file
			//put the best examples in a hash set for quick contains operation
			HashSet<String> set = new HashSet<String>();
			for(int i=0; i<best_examples.length; ++i)
			{
				set.add(best_examples[i]);
			}
			BufferedReader reader = null;
			try
			{
				//open the pool file
				File pool = new File(auxiliary_path + "" + file_separator + "pool.txt");
				//and create a new pool file
				File new_pool = new File(auxiliary_path + "" + file_separator + "pool_intermediate.txt");

				reader = new BufferedReader(new InputStreamReader(
						new FileInputStream(pool), "UTF-8"));
				writer = new PrintWriter(new_pool.getAbsolutePath(), "UTF-8");
				//write all lines from the pool to the new_pool file updated
				String line = reader.readLine();
				while(line != null)
				{
					if(set.contains(line))
					{
						writer.println("true" + line.substring(5, line.length()));
					}
					else
					{
						writer.println(line);
					}
					writer.flush();
					line = reader.readLine();
					counter++;
					bar.progress.setValue(counter);
				}
			}
			catch(Exception e)
			{
				e.printStackTrace();
				JOptionPane.showMessageDialog(null, "Problem occured while writing the pool file.");
			}
			finally
			{
				try{writer.close();}catch(Exception e){e.printStackTrace();}
			}
			
			//copy the intermediate pool file to the pool file
			try
			{
				//open the intermediate pool file
				File intermediate = new File(auxiliary_path + "" + file_separator + "pool_intermediate.txt");
				//open the pool file
				File pool = new File(auxiliary_path + "" + file_separator + "pool.txt");
				reader = new BufferedReader(new InputStreamReader(
						new FileInputStream(intermediate), "UTF-8"));
				writer = new PrintWriter(pool.getAbsolutePath(), "UTF-8");
				//write all lines from the intermediate to the pool file
				String line = reader.readLine();
				while(line != null)
				{
					writer.println(line);
					writer.flush();
					line = reader.readLine();
					counter++;
					bar.progress.setValue(counter);
				}
			}
			catch(Exception e)
			{
				e.printStackTrace();
				JOptionPane.showMessageDialog(null, "Problem occured while writing the pool file.");
			}
			finally
			{
				try{writer.close();}catch(Exception e){e.printStackTrace();}
			}
			train_examples += best_examples.length;
			//nullify the best examples
			best_examples = null;
			bar.dispose();
			return null;
		}
		
		public void done()
		{
			//show the proposal frame for the next examples
			frame = new Active_learning_tool_proposal_window(proposer,
					all_examples-train_examples, train_examples, window, examples_to_propose,
					entropy_plus_diversity, basic_tagset, gain_ratio, k, MVDM);
			frame.init();
		}
		
		private Proposer proposer;
		
	}
	
	private Active_learning_tool_proposal_window frame;
	private String auxiliary_path;
	private int all_examples;
	private int train_examples;
	protected String[] best_examples;
	protected String[] categories;
	private int window;
	private int examples_to_propose;
	private String[] window_examples;
	private boolean entropy_plus_diversity;
	private boolean basic_tagset;
	private boolean gain_ratio;
	private int k;
	private boolean MVDM;
	public static String file_separator;
	
}
