package auxiliary_folder_handling;

import classifier.*;
import gui.active_learning_tool.Active_learning_tool_loading_bar;
import java.util.*;
import javax.swing.*;

public class Selector extends SwingWorker<Void, Void>
{

	//returns the 'new_examples' best examples chosen from the pool
	//according to the state of the knn classifier
	public Selector(Knn k, String[] p, int n, String[] b, Proposer prop, int w)
	{
		knn = k;
		pool = p;
		best_examples = b;
		proposer = prop;
		window = w;
		new_examples = n;
	}
	
	public Void doInBackground()
	{
		best_examples = new String[new_examples];
		//the train set is empty therefore the classifier is null
		if(knn == null)
		{
			//in this case return the first examples
			for(int i=0; i<new_examples; ++i)
			{
				best_examples[i] = pool[i];
			}
			return null;
		}

		//show the loading bar
		Active_learning_tool_loading_bar bar =
			new Active_learning_tool_loading_bar("Selecting Best Examples");
		bar.init();
		bar.progress.setMaximum(pool.length);
		bar.progress.setValue(0);
		
		//sort the incoming examples according to their importance
		TreeMap<Double, ArrayList<String>> examples_sorted = new TreeMap<Double, ArrayList<String>>();
		TreeMap<Double, ArrayList<String>> with_low_frequency = new TreeMap<Double, ArrayList<String>>();
		TreeMap<Double, ArrayList<String>> contained_in_list = new TreeMap<Double, ArrayList<String>>();
		//for each example given
		for(int i=0; i<pool.length; ++i)
		{
			if(knn.is_contained_in_list(get_instance(pool[i], knn)))
			{
				add(contained_in_list, pool[i]);
			}
			else if(pool[i].split(" ")[1].equals("1") || pool[i].split(" ")[1].equals("2"))
			{
				add(with_low_frequency, pool[i]);
			}
			else
			{
				add(examples_sorted, pool[i]);
			}
			bar.progress.setValue(i);
		}
		
		//return the 'new_examples' best examples
		ArrayList<String> list = new ArrayList<String>();
		for(int i=0; i<new_examples; ++i)
		{
			if(list.isEmpty())
			{
				//higher values come first
				if(!examples_sorted.isEmpty())
				{
					list = examples_sorted.pollLastEntry().getValue();
				}
				else if(!with_low_frequency.isEmpty())
				{
					list = with_low_frequency.pollLastEntry().getValue();
				}
				else
				{
					list = contained_in_list.pollLastEntry().getValue();
				}
			}
			best_examples[i] = list.remove(0);
		}
		bar.dispose();
		return null;
	}
	
	public void done()
	{
		proposer.best_examples = best_examples;
		proposer.request_to_propose(window, new_examples);
	}
	
	private void add(TreeMap<Double, ArrayList<String>> map, String pool_line)
	{
		//measure its uncertainty
		double uncertainty = knn.get_uncertainty_for_classification(get_instance(pool_line, knn));
		//and update the map
		if(map.containsKey(uncertainty))
		{
			map.get(uncertainty).add(pool_line);
		}
		else
		{
			ArrayList<String> temp = new ArrayList<String>();
			temp.add(pool_line);
			map.put(uncertainty, temp);
		}
	}
	
	//returns an instance from a pool line using the knn to find out the ambitag values
	public Instance get_instance(String example, Knn knn)
	{
		String[] parts = example.split(" ");
		Instance temp = new Instance();
		for(int i=0; i<18; ++i)
		{
			temp.set_property_value(i, extract_property_value(i, parts, knn));
		}
		return temp;
	}
	
	//returns the value of the index property
	public String extract_property_value(int property_index, String[] parts, Knn knn)
	{
		String word = parts[4];
		switch(property_index)
		{
			//word ending
			case 0:
			{
				if(word.equals("null"))
				{
					return "null";
				}
				//if the word is less than 3 characters long
				if(word.length()<=3)
				{//its ending is the word itself
					return word;
				}
				//else return the three last characters
				return word.substring(
						word.length()-3,
						word.length());
			}
			//the length of the word
			case 1:
			{
				if(word.equals("null"))
				{
					return "0";
				}
				return (new Integer(word.length())).toString();
			}
			//if the word has apostrophe
			case 2:
			{
				if(word.contains("\'"))
				{
					return "1";
				}
				return "0";
			}
			//if the word has digit
			case 3:
			{
				//for each character of the word
				for(int i=0; i<word.length(); ++i)
				{//if it is a digit
					if(Character.isDigit(word.charAt(i)))
					{
						return "1";
					}
				}
				return "0";
			}
			//if the word has dot
			case 4:
			{
				if(word.contains("."))
				{
					return "1";
				}
				return "0";
			}
			//if the word has comma
			case 5:
			{
				if(word.contains(","))
				{
					return "1";
				}
				return "0";
			}
			//if the word has latin character(that is A-Z or a-z)
			case 6:
			{
				if(word.equals("null"))
				{
					return "0";
				}
				//for each character of the word
				for(int i=0; i<word.length(); ++i)
				{//if it is a latin character
					int num_code = (int)word.charAt(i);
					if((num_code>=65 && num_code<=90) || (num_code>=97 && num_code<=122))
					{
						return "1";
					}
				}
				return "0";
			}
			//the ambitag of the word
			case 7:
			{
				return knn.get_ambitag(word);
			}
			//the ending of the next word
			case 8:
			{
				if(parts[5].equals("null"))
				{
					return "null";
				}
				//if the next word is less than 3 characters long
				if(parts[5].length()<=3)
				{//its ending is the word itself
					return parts[5];
				}
				//else return the three last characters
				return parts[5].substring(
						parts[5].length()-3,
						parts[5].length());
			}
			//the ambitag of the next word
			case 9:
			{
				return knn.get_ambitag(parts[5]);
			}
			//the word before the previous word
			case 10:
			{
				return parts[2];
			}
			//the ending of the word before the previous word
			case 11:
			{
				if(parts[2].equals("null"))
				{
					return "null";
				}
				//if the word is less than 3 characters long
				if(parts[2].length()<=3)
				{//its ending is the word itself
					return parts[2];
				}
				//else return the three last characters
				return parts[2].substring(parts[2].length()-3, parts[2].length());
			}
			//the ending of the previous word
			case 12:
			{
				if(parts[3].equals("null"))
				{
					return "null";
				}
				//if the word is less than 3 characters long
				if(parts[3].length()<=3)
				{//its ending is the word itself
					return parts[3];
				}
				//else return the three last characters
				return parts[3].substring(parts[3].length()-3, parts[3].length());
			}
			//the previous word
			case 13:
			{
				return parts[3];
			}
			//the ambitag of the previous word
			case 14:
			{
				return knn.get_ambitag(parts[3]);
			}
			//the category
			case 15:
			{
				return "?";
			}
			//the word itself
			case 16:
			{
				return parts[4];
			}
			//the next word
			case 17:
			{
				return parts[5];
			}
			default:return "null";
		}
	}
	
	private Knn knn;
	//all the examples
	private String[] pool;
	//the selected examples
	private String[] best_examples;
	private Proposer proposer;
	//this variable is used by the proposer
	private int window;
	//how many examples to select
	private int new_examples;
	
}
