package classifier;

import java.util.*;
import converter.*;

public class Net
{
	
	public void train(ArrayList<Instance> knowledge)
	{
		basic_tagset_used = true;
		//check all instances and create the lists
		for(int i=0; i<knowledge.size(); ++i)
		{
			String word = knowledge.get(i).get_property_value(16);
			String category = Converter.reverse(knowledge.get(i).get_category());
			if(!category.contains("/--/--/--/--"))
			{
				basic_tagset_used = false;
			}
			if(category.equals("article/definite/nominative/masculine/singular"))
			{
				articles_definite_nominative_masculine_singular.add(word);
			}
			else if(category.equals("article/definite/nominative/feminine/singular"))
			{
				articles_definite_nominative_feminine_singular.add(word);
			}
			else if(category.equals("punctuation/--/--/--/--"))
			{
				punctuations.add(word);
			}
			else if(category.equals("other/abbreviation/--/--/--"))
			{
				abbreviations.add(word);
			}
			else if(category.equals("other/foreign_word/--/--/--"))
			{
				if(all_lower_case_greek(word) && word.length() >= 3)
				{
					foreign_words.add(word);
				}
				else if(contains_latin_character(word))
				{
					foreign_words.add(word);
				}
			}
			else if(category.equals("other/symbol/--/--/--"))
			{
				symbols.add(word);
			}
			else if(category.equals("other/other/--/--/--"))
			{
				others.add(word);
			}
			//in the following categories we keep words whose characters are not all capitals
			else if(all_upper_case_greek(word))
			{
				//skip next checks
			}
			else if(category.equals("conjunction/--/--/--/--"))
			{
				conjunctions.add(word);
			}
			else if(category.equals("particle/--/--/--/--"))
			{
				particles.add(word);
			}
			else if(category.equals("pronoun/inflectionless/--/--/--"))
			{
				pronouns_inflectionless.add(word);
			}
			else if(category.equals("article/prepositional/genitive/feminine/singular"))
			{
				articles_prepositional_genitive_feminine_singular.add(word);
			}
			else if(category.equals("article/prepositional/accusative/feminine/singular"))
			{
				articles_prepositional_accusative_feminine_singular.add(word);
			}
			else if(category.equals("article/prepositional/accusative/feminine/plural"))
			{
				articles_prepositional_accusative_feminine_plural.add(word);
			}
			else if(category.equals("article/prepositional/accusative/masculine/plural"))
			{
				articles_prepositional_accusative_masculine_plural.add(word);
			}
		}
	}
	
	public String classify(Instance in)
	{
		//the instance's word
		String word = in.get_property_value(16);
		if(articles_definite_nominative_masculine_singular.contains(word))
		{
			return "article/definite/nominative/masculine/singular";
		}
		else if(articles_definite_nominative_feminine_singular.contains(word))
		{
			return "article/definite/nominative/feminine/singular";
		}
		else if(articles_prepositional_genitive_feminine_singular.contains(word))
		{
			return "article/prepositional/genitive/feminine/singular";
		}
		else if(articles_prepositional_accusative_feminine_singular.contains(word))
		{
			return "article/prepositional/accusative/feminine/singular";
		}
		else if(articles_prepositional_accusative_feminine_plural.contains(word))
		{
			return "article/prepositional/accusative/feminine/plural";
		}
		else if(articles_prepositional_accusative_masculine_plural.contains(word))
		{
			return "article/prepositional/accusative/masculine/plural";
		}
		else if(punctuations.contains(word))
		{
			return "punctuation/--/--/--/--";
		}
		else if(conjunctions.contains(word))
		{
			return "conjunction/--/--/--/--";
		}
		else if(particles.contains(word))
		{
			return "particle/--/--/--/--";
		}
		else if(pronouns_inflectionless.contains(word))
		{
			return "pronoun/inflectionless/--/--/--";
		}
		else if(abbreviations.contains(word))
		{
			return "other/abbreviation/--/--/--";
		}
		else if(foreign_words.contains(word))
		{
			return "other/foreign_word/--/--/--";
		}
		else if(symbols.contains(word))
		{
			return "other/symbol/--/--/--";
		}
		else if(others.contains(word))
		{
			return "other/other/--/--/--";
		}
		boolean all_digits = true;
		//for each character of the word
		for(int i=0; i<word.length(); ++i)
		{//if it is a latin character
			int num_code = (int)word.charAt(i);
			if((num_code>=65 && num_code<=90) || (num_code>=97 && num_code<=122))
			{
				if(basic_tagset_used)
				{
					return "other/--/--/--/--";
				}
				return "other/foreign_word/--/--/--";
			}
			//if its is not a digit
			if(!Character.isDigit(word.charAt(i)))
			{
				all_digits = false;
			}
		}
		if(all_digits)
		{
			return "numeral/--/--/--/--";
		}
		return null;
	}
	
	private static boolean all_upper_case_greek(String s)
	{
		for(int i=0; i<s.length(); ++i)
		{
			if(!(913 <= (int)s.charAt(i) && (int)s.charAt(i) <= 937))
			{
				return false;
			}
		}
		return true;
	}
	
	private static boolean all_lower_case_greek(String s)
	{
		for(int i=0; i<s.length(); ++i)
		{
			if(!(940 <= (int)s.charAt(i) && (int)s.charAt(i) < 975))
			{
				return false;
			}
		}
		return true;
	}
	
	private static boolean contains_latin_character(String s)
	{
		for(int i=0; i<s.length(); ++i)
		{
			if((65 <= (int)s.charAt(i) && (int)s.charAt(i) <= 90)
					||
					(97 <= (int)s.charAt(i) && (int)s.charAt(i) <= 122))
			{
				return true;
			}
		}
		return false;
	}
	
	public TreeSet<String> articles_definite_nominative_masculine_singular = new TreeSet<String>();
	public TreeSet<String> articles_definite_nominative_feminine_singular = new TreeSet<String>();
	public TreeSet<String> articles_prepositional_genitive_feminine_singular = new TreeSet<String>();
	public TreeSet<String> articles_prepositional_accusative_feminine_singular = new TreeSet<String>();
	public TreeSet<String> articles_prepositional_accusative_feminine_plural = new TreeSet<String>();
	public TreeSet<String> articles_prepositional_accusative_masculine_plural = new TreeSet<String>();
	public TreeSet<String> punctuations = new TreeSet<String>();
	public TreeSet<String> conjunctions = new TreeSet<String>();
	public TreeSet<String> particles = new TreeSet<String>();
	public TreeSet<String> pronouns_inflectionless = new TreeSet<String>();
	public TreeSet<String> abbreviations = new TreeSet<String>();
	public TreeSet<String> foreign_words = new TreeSet<String>();
	public TreeSet<String> symbols = new TreeSet<String>();
	public TreeSet<String> others = new TreeSet<String>();
	public boolean basic_tagset_used;
}
