package classifier.calculators;

import classifier.*;
import java.util.*;

public class Gain_ratio_calculator
{

	//given a train set calculates the gain ratio of each property
	public double[] get_properties_weight(ArrayList<Instance> knowledge_in)
	{
		knowledge = knowledge_in;
		calculate_categories_entropy();
		calculate_properties_weight();
		return properties_weight;
	}
	
	//finds a number for each property that determines its usefulness
	//in predicting the category of an new unknown instance
	private void calculate_properties_weight()
	{
		properties_weight = new double[knowledge.get(0).get_number_of_properties()];
		for(int i=0; i<properties_weight.length; ++i)
		{
			//use of gain ratio definition
			double property_information_gain = property_information_gain(i);
			double property_values_entropy = property_values_entropy(i);
			if(property_information_gain == 0 || property_values_entropy == 0)
			{
				properties_weight[i] = 0;
			}
			else
			{
				properties_weight[i] = property_information_gain/property_values_entropy;
			}
		}
	}
	
	private double property_information_gain(int property)
	{
		//for each value of the property we hold in which categories it appears and how many
		//times in each of them
		HashMap<String, HashMap<String, Integer>> value_categories =
			new HashMap<String, HashMap<String, Integer>>();
		//for each value of the property we hold the times it appears in the paradigm examples
		HashMap<String, Integer> value_times = new HashMap<String, Integer>();
		Instance instance = null;
		String value = null;
		//for each paradigm example locate the value of the given property
		for(int i=0; i<knowledge.size(); ++i)
		{
			instance = knowledge.get(i);
			value = instance.get_property_value(property);
			//initialize value_categories map
			//hold the categories and times in each category in which it appears
			if(value_categories.containsKey(value))
			{
				HashMap<String, Integer> temp_hash = value_categories.get(value);
				//if for this value the current category has reappeared add 1
				//to the number of appearances for this category
				if(temp_hash.containsKey(instance.get_category()))
				{
					temp_hash.put(instance.get_category(),
					new Integer(temp_hash.get(instance.get_category()).intValue()+1));
				}
				else
				{
					temp_hash.put(instance.get_category(), new Integer(1));
				}
			}
			else
			{
				HashMap<String, Integer> categories_times = new HashMap<String, Integer>();
				categories_times.put(instance.get_category(), new Integer(1));
				value_categories.put(value, categories_times);
			}
			//initialize value_times map
			if(value_times.containsKey(value))
			{
				value_times.put(value,
						value_times.get(value)+1);
			}
			else
			{
				value_times.put(value, new Integer(1));
			}
		}
		//the two maps are now ready
		Iterator<String> it = value_times.keySet().iterator();
		String v = "";
		double sum = 0;
		while(it.hasNext())
		{
			v = it.next();
			double value_possibility = 0;
			double categories_entropy_for_fixed_property_value = 0;
			//calculate value_possibility
			value_possibility = ((double)value_times.get(v))/knowledge.size();
			//calculate categories_entropy_for_fixed_property_value
			HashMap<String, Integer> category_times = value_categories.get(v);
			Iterator<String> it2 = category_times.keySet().iterator();
			while(it2.hasNext())
			{
				String category = it2.next();
				double possibility_for_fixed_value = ((double)category_times.get(category))/
					(double)(value_times.get(v));
				categories_entropy_for_fixed_property_value += possibility_for_fixed_value*
										(Math.log(possibility_for_fixed_value)/Math.log(2));
			}
			categories_entropy_for_fixed_property_value =
				-categories_entropy_for_fixed_property_value;
			//update sum
			sum += value_possibility * categories_entropy_for_fixed_property_value;
		}
		return categories_entropy - sum;
	}
	
	private double property_values_entropy(int property)
	{
		HashMap<String, Integer> values_times = new HashMap<String, Integer>();
		for(int i=0; i<knowledge.size(); ++i)
		{
			String value = knowledge.get(i).get_property_value(property);
			if(values_times.containsKey(value))
			{
				values_times.put(value, new Integer(values_times.get(value).intValue()+1));
			}
			else
			{
				values_times.put(value, new Integer(1));
			}
		}
		double sum = 0;
		Iterator<String> it = values_times.keySet().iterator();
		while(it.hasNext())
		{
			double pos = ((double)values_times.get(it.next()))/((double)knowledge.size());
			sum += pos*(Math.log(pos)/Math.log(2));
		}
		return -sum;
	}
	
	//calculates the entropy of the categories contained in the train set
	private void calculate_categories_entropy()
	{
		//here we store all categories found in the train set and next to each
		//the times we find it
		TreeMap<String, Integer> categories_and_times_of_appearance_in_train_set =
			new TreeMap<String, Integer>();
		String new_category = "";
		for(int i=0; i<knowledge.size(); ++i)
		{
			new_category = knowledge.get(i).get_category();
			if(categories_and_times_of_appearance_in_train_set.containsKey(
					new_category))
			{
				categories_and_times_of_appearance_in_train_set.put(new_category,
				categories_and_times_of_appearance_in_train_set.get(new_category)+1);
			}
			else
			{
				categories_and_times_of_appearance_in_train_set.put(new_category, 1);
			}
		}
		double temp_entropy = 0;
		double temp_possibility = 0;
		String category = "";
		Iterator<String> it =
			categories_and_times_of_appearance_in_train_set.keySet().iterator();
		//we can now calculate the possibility for a category to appear in the train set 
		while(it.hasNext())
		{
			category = it.next();
			temp_possibility = categories_and_times_of_appearance_in_train_set.get(category)
								/(double)knowledge.size();
			//log2(temp_possibility) == ln(temp_possibility)/ln(2)
			temp_entropy += temp_possibility*(Math.log(temp_possibility)/Math.log(2));
		}
		categories_entropy = -temp_entropy;
	}
	
	private double[] properties_weight;
	private ArrayList<Instance> knowledge;
	private double categories_entropy;
	
}
