package classifier.calculators;

import java.util.*;

import classifier.Instance;

public class Word_and_ending_ambitag_calculator
{

	public void calculate_word_and_ending_ambitag(ArrayList<Instance> knowledge_in)
	{
		knowledge = knowledge_in;
		calculate_word_and_ending_ambitag();
	}
	
	public HashMap<String, String> get_word_ambitags()
	{
		return word_ambitag;
	}
	
	public HashMap<String, String> get_ending_ambitags()
	{
		return ending_ambitag;
	}
	
	//according to the given train set of instances we can construct the ambitag
	//of any word or ending contained in it. this way we can know the value of
	//the ambitag property of any new incoming word
	private void calculate_word_and_ending_ambitag()
	{
		HashMap<String, TreeMap<String, Integer>> word_tag_times =
			new HashMap<String, TreeMap<String, Integer>>();
		HashMap<String, TreeMap<String, Integer>> ending_tag_times =
			new HashMap<String, TreeMap<String, Integer>>();
		Instance temp = null;
		//for each instance in the train set
		for(int i=0; i<knowledge.size(); ++i)
		{
			temp = knowledge.get(i);
			//word and its tag
			update_word_and_ending_ambitag(word_tag_times,
					temp.get_property_value(16),
					temp.get_property_value(15));
			//ending of the word and its tag
			update_word_and_ending_ambitag(ending_tag_times,
					temp.get_property_value(0),
					temp.get_property_value(15));
		}
		word_ambitag = finalise_word_and_ending_ambitag(word_tag_times);
		ending_ambitag = finalise_word_and_ending_ambitag(ending_tag_times);
	}
	
	//update the ambitag of the key word or ending adding the tags contained
	//in the ambitag_value variable
	private void update_word_and_ending_ambitag(HashMap<String, TreeMap<String, Integer>>
							word_or_ending_tag_times, String word_or_ending, String tag_value)
	{
		//if word or ending already found
		if(word_or_ending_tag_times.containsKey(word_or_ending))
		{
			//if this tag has been found for this word or ending
			if(word_or_ending_tag_times.get(word_or_ending).containsKey(tag_value))
			{
				//add one the the times of appearances
				word_or_ending_tag_times.get(word_or_ending).put(
						tag_value,
						new Integer(word_or_ending_tag_times.get(word_or_ending).
								get(tag_value).intValue()+1));
			}
			//else, add this tag for this word or ending
			else
			{
				word_or_ending_tag_times.get(word_or_ending).put(tag_value, new Integer(1));
			}
		}
		//else the tag for this word or ending is now created
		else
		{
			TreeMap<String, Integer> temp = new TreeMap<String, Integer>();
			temp.put(tag_value, new Integer(1));
			word_or_ending_tag_times.put(word_or_ending, temp);
		}
	}
	
	//extract the final form of the ambitag value using the collected info
	private HashMap<String, String> finalise_word_and_ending_ambitag(HashMap<String,
			TreeMap<String, Integer>> word_or_ending_tag_times)
	{
		HashMap<String, String> word_or_ending_ambitag = new HashMap<String, String>();
		Iterator<String> it = word_or_ending_tag_times.keySet().iterator();
		//for each word or ending
		while(it.hasNext())
		{
			String word_or_ending = it.next();
			//if this word or ending has been found with one tag only
			if(word_or_ending_tag_times.get(word_or_ending).size() == 1)
			{
				//then its ambitag is this tag
				Iterator<String> it2 = word_or_ending_tag_times.get(word_or_ending).keySet().iterator();
				word_or_ending_ambitag.put(word_or_ending, it2.next());
			}
			//else create the ambitag by sorting the tags according to their number of appearance
			else
			{
				word_or_ending_ambitag.put(word_or_ending,
						get_ambitag(word_or_ending_tag_times.get(word_or_ending)));
			}
		}
		return word_or_ending_ambitag;
	}
	
	//returns the ambitag of the word given its tags and the times each of these tags appears
	private String get_ambitag(TreeMap<String, Integer> tag_times)
	{
		String ambitag = "";
		TreeMap<Integer, String> times_tag = new TreeMap<Integer, String>();
		//a TreeMap keeps its contents sorted
		Iterator<String> it = tag_times.keySet().iterator();
		while(it.hasNext())
		{
			String tag = it.next();
			Integer times = tag_times.get(tag);
			if(times_tag.containsKey(times))
			{
				times_tag.put(times, times_tag.get(times) + "-" + tag);
			}
			else
			{
				times_tag.put(times, tag);
			}
		}
		//a tag comes before another in the ambitag if it appears more times than the other
		//if more than one tags appear the same number of times they are sorted
		//according to their string representation
		while(!times_tag.isEmpty())
		{
			ambitag += times_tag.pollLastEntry().getValue() + "-";
		}
		
		return ambitag.substring(0, ambitag.length()-1);
	}
	
	private ArrayList<Instance> knowledge;
	private HashMap<String, String> word_ambitag;
	private HashMap<String, String> ending_ambitag;
	
}
