package classifier.property_value_difference_metric;

import java.util.*;
import classifier.Instance;

public class Modified_value_difference_metric implements Property_value_difference_metric
{
	
	public Modified_value_difference_metric(ArrayList<Instance> knowledge)
	{
		calculate_property_value_possibility_structure(knowledge);
	}
	
	public double get_difference(int property_index, Instance in, Instance in2)
	{
		String v1 = in.get_property_value(property_index);
		String v2 = in2.get_property_value(property_index);
		if(v1.equals(v2))
		{//distance is 0
			return 0;
		}
		//categories for v2
		HashMap<String, Double> categories_for_v2 =
			property_value_possibility_structure[property_index].get(v2);
		//v2 may not exist in the train set so there could be no categories for it
		//v1 of instance in exists because instance in is contained in the train set
		//- see method Knn.classify(Instance in)
		if(categories_for_v2 == null)
		{//in this case the distance is 1
			return 1;
		}
		//and categories for v1
		HashMap<String, Double> categories_for_v1 =
			property_value_possibility_structure[property_index].get(v1);
		HashSet<String> all_relative_categories = new HashSet<String>();
		all_relative_categories.addAll(categories_for_v1.keySet());
		all_relative_categories.addAll(categories_for_v2.keySet());
		Iterator<String> it = all_relative_categories.iterator();
		
		double value_difference = 0;
		double possibility_for_category_if_value_equals_v1 = 0;
		double possibility_for_category_if_value_equals_v2 = 0;
		String category;
		//for each category found to have value v1 or v2 as the property's value
		while(it.hasNext())
		{
			category = it.next();
			//see if the category was found for the value v1
			if(categories_for_v1.containsKey(category))
			{
				//given the property's value to be v1 what is the possibility that
				//the category is the one we currently check
				possibility_for_category_if_value_equals_v1 = categories_for_v1.get(category);
			}
			else
			{
				possibility_for_category_if_value_equals_v1 = 0;
			}
			//same for value v2
			if(categories_for_v2.containsKey(category))
			{
				//given the property's value to be v2 what is the possibility that
				//the category is the one we currently check
				possibility_for_category_if_value_equals_v2 = categories_for_v2.get(category);
			}
			else
			{
				possibility_for_category_if_value_equals_v2 = 0;
			}
			//modified value difference metric
			value_difference += Math.abs(possibility_for_category_if_value_equals_v1 -
					possibility_for_category_if_value_equals_v2);
		}
		return value_difference;
	}
	
	//the property_value_possibility_structure is used to speed up the use of MVDM
	//this structure is an array of number_of_properties length
	//this array holds a HashMap in each position
	//each HashMap holds an entry for each value of the corresponding property
	//and for every value another HashMap is hold that holds
	//the possibility for each category in the train set given this value
	private void calculate_property_value_possibility_structure(ArrayList<Instance> knowledge)
	{
		property_value_possibility_structure =
			new HashMap[knowledge.get(0).get_number_of_properties()];
		for(int i=0; i<property_value_possibility_structure.length; ++i)
		{
			property_value_possibility_structure[i] =
				new HashMap<String, HashMap<String, Double>>();
		}
		//for each instance in the train set
		Instance temp = null;
		for(int i=0; i<knowledge.size(); ++i)
		{
			temp = knowledge.get(i);
			//for each property of it
			for(int j=0; j<knowledge.get(0).get_number_of_properties(); ++j)
			{
				//get its value and category and update the structure
				update_structure(property_value_possibility_structure[j],
						temp.get_property_value(j),
						temp.get_category());
			}
		}
		finalize_structure();
	}
	
	//given the value of a new property and the category of its instance we update the structure
	private void update_structure(HashMap<String, HashMap<String, Double>> property_map,
			String value, String category)
	{
		//if this value for this property is met for the first time we initialize
		if(!property_map.containsKey(value))
		{
			HashMap<String, Double> new_value_map = new HashMap<String, Double>();
			property_map.put(value, new_value_map);
		}
		//here we take the HashMap of the value of the property
		HashMap<String, Double> value_map = property_map.get(value);
		//if this category is new for this value of this property we initialize
		if(value_map.get(category) == null)
		{
			value_map.put(category, new Double(0));
		}
		//and we add 1 to the numbers of appearance of this category
		value_map.put(category, new Double(value_map.get(category)+1));
	}
	
	//until now we have hold the number of times each category appears for a given value
	//of a property. now we calculate the corresponding possibility that is used in the MVDM
	private void finalize_structure()
	{
		//for every property
		for(int i=0; i<property_value_possibility_structure.length; ++i)
		{
			//and every value of it that appears in the train set
			Iterator<String> it = property_value_possibility_structure[i].keySet().iterator();
			while(it.hasNext())
			{
				String s = it.next();
				//we calculate the corresponding possibility
				fix_value_map(property_value_possibility_structure[i].get(s));
			}
		}
	}
	
	//here we turn times of appearance of each category given a property value into possibilities
	private void fix_value_map(HashMap<String, Double> value_map)
	{
		Iterator<String> it = value_map.keySet().iterator();
		double sum = 0;
		while(it.hasNext())
		{
			sum += value_map.get(it.next());
		}
		it = value_map.keySet().iterator();
		while(it.hasNext())
		{
			String category = it.next();
			value_map.put(category, (double)(value_map.get(category))/sum);
		}
	}
	
	//this structure is used to speed up the use of MVDM
	private HashMap<String, HashMap<String, Double>>[] property_value_possibility_structure;
	
}
