package document;

import java.io.*;
import ipl.sentence_splitter.*;

public class Document_splitter
{
	
	//creates a Token_document from the given file - that is, it parses the text of the file
	//and locates sequences of letters (words) that should be handled differently
	//in the part of speech tagging
	public Token_document get_token_document_for_file(File file)
	{
		Token_document document = new Token_document();
		
		//split the text to its sentences
		Tagger tagger = new Tagger();
		StringBuffer tagged_text =
			new StringBuffer(tagger.mark_file(file.getAbsolutePath()));

		//some variables
		int token_starting_position = 0;
		Token sentence_starting_token = null;
		int cursor_position = 0;
		boolean now_parsing_token = false;
		String token = "";
		Token temp;
		//parse the text character by character
		for(int i=0; i<tagged_text.length(); ++i)
		{
			//the current character
			char current_char = tagged_text.charAt(i);

			//if a white space, a tab or a change_line character is met
			if(current_char == ' ' || current_char == '\r' ||
					current_char == '\n' || current_char == '\t')
			{
				//ignore the \r control character
				if(current_char == '\r')
				{
					continue;
				}
				//if we were parsing a token we've found the end of it
				if(now_parsing_token)
				{
					temp = new Token(token, token_starting_position, cursor_position);
					document.add_token(temp);
					if(sentence_starting_token == null)
					{
						sentence_starting_token = temp;
					}
					now_parsing_token = false;
				}
				cursor_position++;
				//go to the next character
				continue;
			}
			
			//if a comma is met
			if(current_char == ',')
			{
				//if the following character is a white space or a change_line_character
				//then this comma is considered a different token
				if(i < tagged_text.length()-1 && (tagged_text.charAt(i+1) == ' ' ||
						tagged_text.charAt(i+1) == '\r' || tagged_text.charAt(i+1) == '\n'
						|| tagged_text.charAt(i+1) == '\t'))
				{
					//if we were parsing a token we have found its end
					if(now_parsing_token)
					{
						temp = new Token(token, token_starting_position, cursor_position);
						document.add_token(temp);
						if(sentence_starting_token == null)
						{
							sentence_starting_token = temp;
						}
					}
					//also, we have found the comma token
					temp = new Token(",", cursor_position, cursor_position+1);
					document.add_token(temp);
					if(sentence_starting_token == null)
					{
						sentence_starting_token = temp;
					}
					now_parsing_token = false;
					cursor_position++;
					continue;
				}
				//if this comma is not a different token
				else
				{
					if(now_parsing_token)
					{
						token += current_char;
						cursor_position++;
						continue;
					}
					else
					{
						now_parsing_token = true;
						token_starting_position = cursor_position;
						token = current_char+ "";
						cursor_position++;
						continue;
					}
				}
			}
			
			//if a dot is met
			if(current_char == '.')
			{
				//check the marking for an ending of a period
				if(tagged_text.charAt(i+2) == 'y' || i == tagged_text.length()-3)
				{
					//if it is the ending of a period then we have found the ending
					//of the current token
					if(now_parsing_token)
					{
						temp = new Token(token, token_starting_position, cursor_position);
						document.add_token(temp);
						if(sentence_starting_token == null)
						{
							sentence_starting_token = temp;
						}
					}
					//also, we have found the dot token
					temp = new Token(".", cursor_position, cursor_position+1);
					document.add_token(temp);
					if(sentence_starting_token == null)
					{
						sentence_starting_token = temp;
					}
					//and, we have found the ending of a sentence
					document.add_sentence(
							new Token_sentence(sentence_starting_token, temp));
					sentence_starting_token = null;
					now_parsing_token = false;
					//skip the marking
					i += 2;
					cursor_position++;
					continue;
				}
				//if it is not the ending of the current period we continue
				else
				{
					if(now_parsing_token)
					{
						token += current_char;
						//pass the marking
						i += 2;
						cursor_position++;
						continue;
					}
					else
					{
						now_parsing_token = true;
						token_starting_position = cursor_position;
						token = current_char+ "";
						//pass the marking
						i += 2;
						cursor_position++;
						continue;
					}
				}
			}
			
			//if a '!' or a ';' is met
			if(current_char == '!' || current_char == ';')
			{
				//we have found the ending of a token
				if(now_parsing_token)
				{
					temp = new Token(token, token_starting_position, cursor_position);
					document.add_token(temp);
					if(sentence_starting_token == null)
					{
						sentence_starting_token = temp;
					}
				}
				//also, we have found the punctuation token
				temp = new Token(current_char+"", cursor_position, cursor_position+1);
				document.add_token(temp);
				if(sentence_starting_token == null)
				{
					sentence_starting_token = temp;
				}
				//and, we have found the ending of a sentence
				document.add_sentence(
						new Token_sentence(sentence_starting_token, temp));
				sentence_starting_token = null;
				now_parsing_token = false;
				cursor_position++;
				continue;
			}
			
			//special characters
			if(is_special_character(current_char))
			{
				//the token until now
				if(now_parsing_token)
				{
					temp = new Token(token, token_starting_position, cursor_position);
					document.add_token(temp);
					if(sentence_starting_token == null)
					{
						sentence_starting_token = temp;
					}
				}
				//the special character token
				temp = new Token(current_char+"", cursor_position, cursor_position+1);
				document.add_token(temp);
				if(sentence_starting_token == null)
				{
					sentence_starting_token = temp;
				}
				now_parsing_token = false;
				cursor_position++;
				continue;
			}
			
			//if any other character is met
			//if we are parsing a token hold the current character and continue
			if(now_parsing_token)
			{
				token += current_char;
			}
			//else mark the starting of a new token and hold the current character
			else
			{
				now_parsing_token = true;
				token_starting_position = cursor_position;
				token = current_char + "";
			}
			
			//if end of text reached complete the last token
			if(i == tagged_text.length()-1)
			{
				if(now_parsing_token)
				{
					temp = new Token(token, token_starting_position, cursor_position);
					document.add_token(temp);
				}
			}
			cursor_position++;
		}
		return document;
	}
	
	//checks if the the ch character is a special character
	private boolean is_special_character(char ch)
	{
		if(ch == '\u0028' || ch == '\u0029' || ch == '\u0022' || ch == '\u0025' || ch == '\u003c' ||
				ch == '\u003e' || ch == '\u00ab' || ch == '\u00bb' || ch == '\u007b' || ch == '\u007d' ||
				ch == '\u005b' || ch == '\u005d' || ch == '\u002f' || ch == '\\' || ch == '\u002a' || 
				ch == '\u0024' || ch == '\u003a' || ch == '' || ch == '\u00b7' || ch == '\u002d' ||
				ch == '\u002b')
		{
			return true;
		}
		return false;
	}
	
}



