package gui.active_learning_tool;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.io.*;
import java.net.URL;

import auxiliary_folder_handling.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.BufferedImage;

public class Active_learning_tool_proposal_window extends JFrame
{

	public Active_learning_tool_proposal_window(Proposer p, int paradeigms,
			int trained, int window, int examples_to_propose, boolean entropy_plus_diversity,
			boolean basic_tagset, boolean gain_ratio, int k, boolean MVDM)
	{
		//the title
		super("Active Learning Progress");
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setResizable(false);
		setSize(380, 180);
		//appear in the center of the screen
		setLocationRelativeTo(null);
		//set the panel
		setContentPane(create_panel(paradeigms, trained, window, examples_to_propose));
		//set the menubar
		setJMenuBar(create_menu_bar(window, paradeigms, examples_to_propose));
		file_separator = System.getProperty("file.separator");
		file_separator = (file_separator.equals("\\"))?"\\\\":file_separator;
		proposer = p;
		//change the frame icon
		BufferedImage image = null;
        try
        {
        	image = ImageIO.read(new URL("file:"+new File("pics" + file_separator +
        			"icon.JPG").getAbsolutePath()));
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        setIconImage(image);
        //set the advanced options
        e_p_d.setSelected(entropy_plus_diversity);
        k_spinner.setValue(k);
        this.gain_ratio.setSelected(gain_ratio);
        this.MVDM.setSelected(MVDM);
	}
	
	//shows the frame
	public void init()
	{
		setVisible(true);
	}
	
	//creates the menubar
	private JMenuBar create_menu_bar(int window, int paradeigms, int examples_to_propose)
	{
		JMenuBar bar = new JMenuBar();

		//the options menu
		JMenu options = new JMenu("Options");

		//the search window option
		JPanel s_w_panel = new JPanel();
		//initial value, lowest value, highest value, step
		//a value more than 300000 for the window is likely to throw a heap overflow error
		window = (window<=paradeigms)?window:paradeigms;
		SpinnerModel model = new SpinnerNumberModel((window > 300000)?300000:window,
				0, (paradeigms > 300000)?300000:paradeigms, (paradeigms<500)?1:10);
		s_w_spinner = new JSpinner(model);
		s_w_spinner.setPreferredSize(new Dimension(70, 20));
		s_w_panel.add(new JLabel("Search Window"));
		s_w_panel.add(s_w_spinner);
		
		//the batch size option
		JPanel b_s_panel = new JPanel();
		//initial value, lowest value, highest value, step
		examples_to_propose = (examples_to_propose<=paradeigms)?examples_to_propose:paradeigms;
		SpinnerModel model2 =
			new SpinnerNumberModel((examples_to_propose > 300000)?300000:examples_to_propose,
					0, (paradeigms > 300000)?300000:paradeigms, 1);
		b_s_spinner = new JSpinner(model2);
		b_s_spinner.setPreferredSize(new Dimension(70, 20));
		JLabel label = new JLabel("Batch Size");
		label.setPreferredSize(new Dimension(90, 20));
		b_s_panel.add(label);
		b_s_panel.add(b_s_spinner);
		
		options.add(s_w_panel);
		options.add(b_s_panel);
		
		//the advanced menu
		JMenu advanced = new JMenu("Advanced");
		
		//the entropy selection measure option
		entropy = new JRadioButtonMenuItem("Entropy");
		entropy.setSelected(true);
		
		//the entropy plus diversity selection measure option
		e_p_d = new JRadioButtonMenuItem("Entropy plus Diversity");
		
		//the selection measures choices group
	    ButtonGroup selection_measure_choices_group = new ButtonGroup();
	    selection_measure_choices_group.add(entropy);
	    selection_measure_choices_group.add(e_p_d);
		
		JMenu selection_measure = new JMenu("Selection Measure");
		selection_measure.add(entropy);
		selection_measure.add(e_p_d);
		
		JSeparator separator0 = new JSeparator();
		
		//the k option
		JPanel k_panel = new JPanel();
		//initial value, lowest value, highest value, step
		SpinnerModel model3 = new SpinnerNumberModel(1, 1, 20, 1);
		k_spinner = new JSpinner(model3);
		JLabel k = new JLabel("K:");
		k.setPreferredSize(new Dimension(50, 20));
		k_panel.add(k);
		k_panel.add(k_spinner);
		
		//the gain_ratio option
		gain_ratio = new JCheckBoxMenuItem("Gain Ratio");
		gain_ratio.setSelected(true);
		
		//the overlap option
		overlap = new JRadioButtonMenuItem("Overlap");
		overlap.setSelected(true);
		
		//the MVDM option
		MVDM = new JRadioButtonMenuItem("Modified Value");
		MVDM.setSelected(true);
		
		//the difference metric choices group
	    ButtonGroup difference_metric_choices_group = new ButtonGroup();
	    difference_metric_choices_group.add(overlap);
	    difference_metric_choices_group.add(MVDM);
		
		JMenu difference_metric = new JMenu("Difference Metrics");
		difference_metric.add(overlap);
		difference_metric.add(MVDM);
		
		advanced.add(selection_measure);
		advanced.add(separator0);
		advanced.add(k_panel);
		advanced.add(gain_ratio);
		advanced.add(difference_metric);
		
		bar.add(options);
		bar.add(advanced);
		
		return bar;
	}
	
	//creates the panel of the proposal window
	private JPanel create_panel(int paradeigms, int trained, int window, int examples_to_propose)
	{
		JPanel panel = new JPanel();
		
		panel.setBackground(new Color(239, 239, 239));
		
		//void label
		JLabel label0 = new JLabel(" ");
		label0.setPreferredSize(new Dimension(340, 10));
		
		//the available examples
		JLabel label1 = new JLabel("Remaining Canditate Examples:");
		label1.setPreferredSize(new Dimension(260, 20));
		
		JLabel label2 = new JLabel(paradeigms+"");
		label2.setPreferredSize(new Dimension(70, 20));
		
		JLabel label3 = new JLabel("Examples Trained On:");
		label3.setPreferredSize(new Dimension(260, 20));
		
		JLabel label4 = new JLabel(trained+"");
		label4.setPreferredSize(new Dimension(70, 20));
		
		//void label
		JLabel label5 = new JLabel(" ");
		label5.setPreferredSize(new Dimension(340, 10));
		
		//the propose button
		JButton button = new JButton("New Batch of Training Examples");
		button.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent event)
			{
				if((Integer)s_w_spinner.getValue() < (Integer)b_s_spinner.getValue())
				{
					JOptionPane.showMessageDialog(null,
							"The batch size can not be larger than\n" +
							"the size of the search window.");
				}
				else if((Integer)b_s_spinner.getValue() == 0)
				{
					JOptionPane.showMessageDialog(null,
							"The batch size must not be set to zero.");
				}
				else
				{
					proposer.save_parameters(!entropy.isSelected(), (Integer)k_spinner.getValue(),
							gain_ratio.isSelected(), !overlap.isSelected());
					proposer.set_spinner_values((Integer)s_w_spinner.getValue(),
							(Integer)b_s_spinner.getValue());
					proposer.request_to_propose((Integer)s_w_spinner.getValue(),
							(Integer)b_s_spinner.getValue());
				}
			}
		});
		
		//the save train file button
		JButton train_button = new JButton("Save Train File");
		train_button.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent event)
			{
				JFileChooser file_chooser = new JFileChooser();
				file_chooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				if(file_chooser.showDialog(null, "Save") == JFileChooser.APPROVE_OPTION)
				{
					//if a file is selected we handle it
					if(file_chooser.getSelectedFile().getAbsolutePath() != null)
					{
						proposer.request_to_save_train_file(file_chooser.getSelectedFile());
					}
				}
			}
		});
		
		//adding the parts
		panel.add(label0);
		panel.add(label1);
		panel.add(label2);
		panel.add(label3);
		panel.add(label4);
		panel.add(label5);
		panel.add(train_button);
		panel.add(button);
		
		return panel;
	}
	
	private Proposer proposer;
	private JRadioButtonMenuItem entropy;
	private JRadioButtonMenuItem e_p_d;
	private JSpinner s_w_spinner;
	private JSpinner b_s_spinner;
	private JSpinner k_spinner;
	private JCheckBoxMenuItem gain_ratio;
	private JRadioButtonMenuItem overlap;
	private JRadioButtonMenuItem MVDM;
	public static String file_separator;
	
}
