package gui.annotation_tool;

import java.awt.event.*;
import java.awt.image.BufferedImage;

import javax.imageio.ImageIO;
import javax.swing.*;
import java.io.*;
import java.net.URL;

public class Annotation_tool_frame extends JFrame
{

	public Annotation_tool_frame(Annotation_tool_gui_manager gui_manager)
	{
		//the title
		super("Annotation Tool");
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setResizable(false);
		setSize(550, 590);
		//appear in the center of the screen
		setLocationRelativeTo(null);
		file_separator = System.getProperty("file.separator");
		file_separator = (file_separator.equals("\\"))?"\\\\":file_separator;
		manager = gui_manager;
		bar = construct_menu();
		setJMenuBar(bar);
		panel = new Annotation_tool_panel(manager);
		getContentPane().add(new JLabel(new ImageIcon("." + file_separator + "pics" +
				file_separator + "ann.JPG")));
		//change the frame icon
		BufferedImage image = null;
        try
        {
        	image = ImageIO.read(new URL("file:"+new File("pics" + file_separator +
        			"icon.JPG").getAbsolutePath()));
        }
        catch (IOException e)
        {
            e.printStackTrace();
        }
        setIconImage(image);
	}
	
	//constructs the menu bar for the frame
	private JMenuBar construct_menu()
	{
		//the menu bar
		JMenuBar bar = new JMenuBar();
		
		//the file menu
		JMenu file = new JMenu("File");
		
		//the open_text menu item
		JMenuItem open_text = new JMenuItem("Open Text...");
		open_text.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent event)
			{
				//when the 'open text' choice is pressed a file chooser appears
				JFileChooser file_chooser = new JFileChooser();
				file_chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
				if(file_chooser.showDialog(null, "Open Text") == JFileChooser.APPROVE_OPTION)
				{
					//if a file is selected we handle it
					if(file_chooser.getSelectedFile().getAbsolutePath() != null)
					{
						manager.request_to_open_text(file_chooser.getSelectedFile());
					}
				}
			}
		});
		
		//the file separator No 1
		JSeparator file_separator1 = new JSeparator();
		
		//the open_draft menu item
		JMenuItem open_draft = new JMenuItem("Open Draft...");
		open_draft.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent event)
			{
				//when the 'open draft' choice is pressed a file chooser appears
				JFileChooser file_chooser = new JFileChooser();
				file_chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
				if(file_chooser.showDialog(null, "Open Draft") == JFileChooser.APPROVE_OPTION)
				{
					//if a file is selected we handle it
					if(file_chooser.getSelectedFile().getAbsolutePath() != null)
					{
						manager.request_to_open_draft(file_chooser.getSelectedFile());
					}
				}
			}
		});
		
		//the save_draft menu item
		JMenuItem save_draft = new JMenuItem("Save Draft...");
		save_draft.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent event)
			{
				//when the 'save draft' choice is pressed a file chooser appears
				JFileChooser file_chooser = new JFileChooser();
				if(file_chooser.showDialog(null, "Save Draft") == JFileChooser.APPROVE_OPTION)
				{
					//if a file is selected we handle it
					if(file_chooser.getSelectedFile().getAbsolutePath() != null)
					{
						manager.request_to_save_draft(file_chooser.getSelectedFile());
					}
				}
			}
		});
		
		//the file separator No 2
		JSeparator file_separator2 = new JSeparator();
		
		//the save_as_train_file menu item
		JMenuItem save_as_train_file = new JMenuItem("Save As Train File...");
		save_as_train_file.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent event)
			{
				//when the 'save as train file' choice is pressed a file chooser appears
				JFileChooser file_chooser = new JFileChooser();
				if(file_chooser.showDialog(null, "Save Train File") == JFileChooser.APPROVE_OPTION)
				{
					//if a file is selected we handle it
					if(file_chooser.getSelectedFile().getAbsolutePath() != null)
					{
						manager.request_to_save_as_train_file(file_chooser.getSelectedFile());
					}
				}
			}
		});
		
		//the save_as_xml menu item
		JMenuItem save_as_xml = new JMenuItem("Save As XML...");
		save_as_xml.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent event)
			{
				//when the 'save as xml' choice is pressed a file chooser appears
				JFileChooser file_chooser = new JFileChooser();
				if(file_chooser.showDialog(null, "Save XML File") == JFileChooser.APPROVE_OPTION)
				{
					//if a file is selected we handle it
					if(file_chooser.getSelectedFile().getAbsolutePath() != null)
					{
						manager.request_to_save_as_xml(file_chooser.getSelectedFile());
					}
				}
			}
		});

		//the file separator No 3
		JSeparator file_separator3 = new JSeparator();
		
		//the exit menu item
		JMenuItem exit = new JMenuItem("Exit");
		exit.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent event)
			{
				manager.request_to_exit();
			}
		});
		
		//the options menu
		JMenu options = new JMenu("Options");
		
		//the high_tag_detail menu item
		high_tag_detail = new JCheckBoxMenuItem("Extended Tagset");
		high_tag_detail.setSelected(true);
		high_tag_detail.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent event)
			{
				manager.tag_detail_changed();
			}
		});
		
		//the options separator No1
		JSeparator options_separator1 = new JSeparator();
		
		//the normalize_text menu item
		normalize_text = new JCheckBoxMenuItem("Normalize Text");
		normalize_text.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent event)
			{
				manager.normalization_option_changed();
			}
		});
		
		JMenu auto_tag = new JMenu("AutoTag");
		
		//the auto tag separator
		JSeparator auto_tag_separator = new JSeparator();
		
		//the tag_'em menu item
		tag_em = new JMenuItem("Tag Current Document");
		tag_em.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent event)
			{
				manager.request_to_tag();
			}
		});
		
		//the tag_em_all menu item
		tag_em_all = new JMenuItem("Tag All Documents");
		tag_em_all.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent event)
			{
				manager.request_to_tag_all();
			}
		});
		
		//the classifier
		JMenu classifier = new JMenu("Advanced");
		
		//the train_set_choice menu
		JMenu train_set_choice = new JMenu("Train Set");
		
		//the default_train_set menu item
		default_train_set = new JRadioButtonMenuItem("Default Train Set");
		default_train_set.setSelected(true);
		
		//the custom_train_set menu item
		custom_train_set = new JRadioButtonMenuItem("Custom Train Set...");
		custom_train_set.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent event)
			{
				//when the 'custom train set' choice is pressed a file chooser appears
				JFileChooser file_chooser = new JFileChooser();
				file_chooser.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
				if(file_chooser.showDialog(null, "Select") == JFileChooser.APPROVE_OPTION)
				{
					//if a file is selected we handle it
					if(file_chooser.getSelectedFile().getAbsolutePath() != null)
					{
						custom_train_file = file_chooser.getSelectedFile();
					}
				}
				//the custom choice was canceled
				else
				{
					default_train_set.setSelected(true);
				}
			}
		});
		
		//the train set choices group
	    ButtonGroup train_set_choices_group = new ButtonGroup();
	    train_set_choices_group.add(default_train_set);
	    train_set_choices_group.add(custom_train_set);
		
		//the classifier separator
		JSeparator classifier_separator = new JSeparator();
		
		//the gain_ratio
		gain_ratio = new JCheckBoxMenuItem("Gain Ratio");
		gain_ratio.setSelected(true);
		
		//the overlap difference metric
		overlap = new JRadioButtonMenuItem("Overlap");
		overlap.setSelected(true);
		
		//the modified_value difference metric
		modified_value = new JRadioButtonMenuItem("Modified Value");
		
		
		//the difference metric choices group
	    ButtonGroup difference_metric_choices_group = new ButtonGroup();
	    difference_metric_choices_group.add(overlap);
	    difference_metric_choices_group.add(modified_value);
		
	    //the difference metrics JMenu
	    JMenu difference_metrics = new JMenu("Difference Metrics");
	    difference_metrics.add(overlap);
	    difference_metrics.add(modified_value);
	    
		//the k_spinner
		JPanel spinner_panel = new JPanel();
		//initial value, lowest value, highest value, step
		SpinnerModel model = new SpinnerNumberModel(1, 1, 20, 1);
		k_spinner = new JSpinner(model);
		spinner_panel.add(new JLabel("K:      "));
		spinner_panel.add(k_spinner);
		
		//classifier separator No2
		JSeparator classifier_separator_2 = new JSeparator();
		
		//gluing the components
		file.add(open_text);
		file.add(file_separator1);
		file.add(open_draft);
		file.add(save_draft);
		file.add(file_separator2);
		file.add(save_as_train_file);
		file.add(save_as_xml);
		file.add(file_separator3);
		file.add(exit);
		options.add(normalize_text);
		options.add(options_separator1);
		options.add(high_tag_detail);
		auto_tag.add(tag_em_all);
		auto_tag.add(auto_tag_separator);
		auto_tag.add(tag_em);
		train_set_choice.add(default_train_set);
		train_set_choice.add(custom_train_set);
		classifier.add(train_set_choice);
		classifier.add(classifier_separator);
		classifier.add(spinner_panel);
		classifier.add(classifier_separator_2);
		classifier.add(gain_ratio);
		classifier.add(difference_metrics);
		
		//adding the components
		bar.add(file);
		bar.add(options);
		bar.add(auto_tag);
		bar.add(classifier);
		return bar;
	}
	
	protected Annotation_tool_panel panel;
	protected JMenuItem tag_em_all;
	protected JMenuItem tag_em;
	protected JRadioButtonMenuItem default_train_set;
	protected JRadioButtonMenuItem custom_train_set;
	protected JCheckBoxMenuItem high_tag_detail;
	protected JCheckBoxMenuItem normalize_text;
	protected JCheckBoxMenuItem gain_ratio;
	protected JSpinner k_spinner;
	protected File custom_train_file;
	protected Annotation_tool_gui_manager manager;
	protected JRadioButtonMenuItem overlap;
	protected JRadioButtonMenuItem modified_value;
	protected JMenuBar bar;
	public static String file_separator;
	
}
