package gui.annotation_tool;

import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.text.*;
import document.*;
import classifier.*;
import instance_provider.*;

public class Annotation_tool_gui_manager
{
	//the sole constructor
	public Annotation_tool_gui_manager()
	{
		frame = new Annotation_tool_frame(this);
		frame.panel.setVisible(false);
		open_files = new String[0][0];
		open_documents = new Token_document[0];
		file_separator = System.getProperty("file.separator");
		file_separator = (file_separator.equals("\\"))?"\\\\":file_separator;
		default_train_file_high_detail = "." + file_separator + "texts" + file_separator + "high_train.train";
		default_train_file_low_detail = "." + file_separator + "texts" + file_separator + "low_train.train";
		
		current_file_index = -1;
		current_token = new Token("\'unknown\'", -1, -1);
		now_tagging = false;
		//the highlight colors
		tag_to_color = new TreeMap<String, Color>();
		tag_to_color.put("--", new Color(200, 200, 200));
		tag_to_color.put("article", new Color(210, 240, 230));
		tag_to_color.put("verb", new Color(210, 230, 190));
		tag_to_color.put("punctuation", new Color(210, 230, 230));
		tag_to_color.put("adjective", new Color(230, 190, 210));
		tag_to_color.put("adverb", new Color(230, 210, 230));
		tag_to_color.put("conjuction", new Color(230, 230, 210));
		tag_to_color.put("noun", new Color(240, 240, 200));
		tag_to_color.put("numeral", new Color(240, 200, 240));
		tag_to_color.put("particle", new Color(240, 200, 200));
		tag_to_color.put("preposition", new Color(200, 190, 240));
		tag_to_color.put("pronoun", new Color(200, 240, 200));
		tag_to_color.put("other", new Color(250, 200, 190));
	}
	
	//initiates the graphic user interface
	public void init()
	{
		frame.setVisible(true);
	}
	
	//handles the request to open a text
	protected void request_to_open_text(File file)
	{
		if(now_tagging)
		{
			JOptionPane.showMessageDialog(null,
					"Tagging operation in progress. Please wait.");
			return;
		}
		frame.normalize_text.setSelected(false);
		normalization_option_changed();
		//open the requested files
		File[] files;
		if(file.isFile())
		{
			files = new File[]{file};
		}
		else
		{
			files = file.listFiles();
		}
		//if too many files are selected perhaps the operation should not continue
		if(files.length > 50)
		{
			if(JOptionPane.showConfirmDialog(null, "Too many text files are contained in " +
					"the selected folder.\n" +
					"It is recommended to abandon the operation and open fewer files.\n" +
					"Continue anyway?") != JOptionPane.OK_OPTION)
			{
				return;
			}
		}
		//check if the files have the correct format
		for(int i=0; i<files.length; ++i)
		{
			if(!files[i].getName().substring(files[i].getName()
					.length()-3).equalsIgnoreCase("txt"))
			{
				JOptionPane.showMessageDialog(null, "Unable to open file " + files[i].getName() +
						".\nText files must be of type .txt.");
				return;
			}
		}
		//create a different document for each file
		open_documents = new Token_document[files.length];
		//keep the text of each file
		open_files = new String[files.length][2];
		for(int i=0; i<files.length; ++i)
		{
			//the document
			open_documents[i] =
				new Document_splitter().get_token_document_for_file(files[i]);
			//the name of the file
			open_files[i][0] = files[i].getName();
			//the text of the file
			String text = "";
			try
			{
				BufferedReader reader = new BufferedReader(new InputStreamReader(
				new FileInputStream(files[i]), "UTF-8"));
				String line = "";
				line = reader.readLine();
				while(line != null)
				{
					text += line + "\n";
					line = reader.readLine();
				}
			}
			catch(IOException e)
			{
				e.printStackTrace();
			}
		    //if first character is not a text character remove it
		    if((int)text.charAt(0) == 65279)
		    {
		    	text = text.substring(1, text.length());
		    }
			open_files[i][1] = text;
			//add dummy token to empty texts
			if(text.equals(""))
			{
				open_documents[i].add_token(new Token("", 0, 0));
			}
		}
		//starts from the first file
		current_file_index = 0;
		//update the user interface
		if(!(frame.getContentPane() instanceof Annotation_tool_panel))
		{
			frame.setContentPane(frame.panel);
		}
		update_text();
		update_tag_panel();
		tag_detail_changed();
		update_document_bar_message();

		frame.panel.setVisible(true);
	}
	
	//handles the request to open a draft
	protected void request_to_open_draft(File file)
	{
		if(now_tagging)
		{
			JOptionPane.showMessageDialog(null,
					"Tagging operation in progress. Please wait.");
			return;
		}
		frame.normalize_text.setSelected(false);
		normalization_option_changed();
		if(!file.getName().substring(file.getName().length()-5).equalsIgnoreCase("draft"))
		{
			JOptionPane.showMessageDialog(null,
				"The selected file should be of type \'.draft\'.");
			return;
		}
		//check if the file has the correct format
		if(!file.getName().substring(file.getName()
					.length()-5).equalsIgnoreCase("draft"))
		{
			JOptionPane.showMessageDialog(null, "Unable to open file " + file.getName() +
						".\nDraft files must be of type .draft.");
			return;
		}
		try
		{
			ObjectInputStream reader = new ObjectInputStream(new FileInputStream(file));
			open_files = (String[][])reader.readObject();
			open_documents = (Token_document[])reader.readObject();
		}
		catch(FileNotFoundException e)
		{
			JOptionPane.showMessageDialog(null,
				"The selected file does not exist.");
			e.printStackTrace();
		}
		catch(IOException e)
		{
			JOptionPane.showMessageDialog(null,
				"Error occured while reading the file.");
			e.printStackTrace();
		}
		catch(ClassNotFoundException e)
		{
			e.printStackTrace();
		}
		//starts from the first file
		current_file_index = 0;
		//update the user interface
		if(!(frame.getContentPane() instanceof Annotation_tool_panel))
		{
			frame.setContentPane(frame.panel);
		}
		update_text();
		update_highlights();
		update_tag_panel();
		tag_detail_changed();
		update_document_bar_message();
		frame.panel.setVisible(true);
	}
	
	//stores the open_files and the open_documents arrays
	protected void request_to_save_draft(File file)
	{
		if(now_tagging)
		{
			JOptionPane.showMessageDialog(null,
					"Tagging operation in progress. Please wait.");
			return;
		}
		if(current_file_index == -1)
		{
			JOptionPane.showMessageDialog(null,
				"No files are currently open.");
			return;
		}
		frame.normalize_text.setSelected(false);
		normalization_option_changed();
		try
		{
			//the new file is created
			ObjectOutputStream writer = new ObjectOutputStream(
					new FileOutputStream(file.getAbsolutePath() + ".draft"));
			//and written
			writer.writeObject(open_files);
			writer.writeObject(open_documents);
			writer.close();
		}
		catch(FileNotFoundException e)
		{
			e.printStackTrace();
		}
		catch(IOException e)
		{
			JOptionPane.showMessageDialog(null,
				"Problem occured while saving the draft.");
			e.printStackTrace();
		}
	}
	
	//stores the annotated tokens in a train file
	protected void request_to_save_as_train_file(File file)
	{
		if(now_tagging)
		{
			JOptionPane.showMessageDialog(null,
					"Tagging operation in progress. Please wait.");
			return;
		}
		if(current_file_index == -1)
		{
			JOptionPane.showMessageDialog(null,
				"No files are currently open.");
			return;
		}
		try
		{
			PrintWriter writer =
				new PrintWriter(file.getAbsolutePath() + ".train", "UTF-8");
			//for every document
			for(int i=0; i<open_documents.length; ++i)
			{
				String line = "";
				Token_document document = open_documents[i];
				//four temporary tokens
				Token pre_previous = null;
				Token previous = null;
				Token current = document.get_token_in_cursor_position(0);
				Token next = document.get_next_token(current);
				//for every token in it
				while(current != next)
				{
					//print the train info
					//if the current token has been tagged
					if(!current.get_tag().equals("--/--/--/--/--"))
					{
						//the pre_previous word
						if(pre_previous != null)
						{
							line += pre_previous.get_token() + " ";
						}
						else
						{
							line += "null ";
						}
						//the previous word
						if(previous != null)
						{
							line += previous.get_token() + " ";
						}
						else
						{
							line += "null ";
						}
						line += current.get_token() + " " +
								current.get_tag() + " " + next.get_token();
					}
					if(!line.equals(""))
					{
						writer.println(line);
						writer.flush();
						line = "";
					}
					//go to next token
					pre_previous = previous;
					previous = current;
					current = next;
					next = document.get_next_token(current);
				}
				//print the last token
				if(!current.get_tag().equals("--/--/--/--/--"))
				{
					//the pre_previous word
					if(pre_previous != null)
					{
						line += pre_previous.get_token() + " ";
					}
					else
					{
						line += "null ";
					}
					//the previous word
					if(previous != null)
					{
						line += previous.get_token() + " ";
					}
					else
					{
						line += "null ";
					}
					line += current.get_token() + " " +
							current.get_tag() + " null";
				}
				if(!line.equals(""))
				{
					writer.println(line);
					writer.flush();
					line = "";
				}
			}
			writer.close();
		}
		catch(IOException e)
		{
			e.printStackTrace();
		}
	}
	
	//stores the open documents in an xml file
	protected void request_to_save_as_xml(File file)
	{
		if(now_tagging)
		{
			JOptionPane.showMessageDialog(null,
					"Tagging operation in progress. Please wait.");
			return;
		}
		if(current_file_index == -1)
		{//no file is yet loaded
			return;
		}
		try
		{
			PrintWriter writer =
				new PrintWriter(file.getAbsolutePath() + ".xml", "UTF-8");
			writer.println("<folder>");
			//for every document
			for(int i=0; i<open_documents.length; ++i)
			{
				//get the document
				Token_document document = open_documents[i];
				String temp_file_name = open_files[i][0];
				temp_file_name = temp_file_name.replace("&", "&amp;");
				temp_file_name = temp_file_name.replace("<", "&lt;");
				temp_file_name = temp_file_name.replace(">", "&gt;");
				temp_file_name = temp_file_name.replace("\'", "&apos;");
				temp_file_name = temp_file_name.replace("\"", "&quot;");
				writer.println("<document file=\"" + temp_file_name + "\">");
				Token_sentence sentence = document.get_first_sentence();
				//for every sentence in the document
				while(sentence != null)
				{
					writer.println("<sentence>");
					Token first = sentence.get_first_token();
					Token last = sentence.get_last_token();
					Token current = first;
					//for every token in the sentence
					while(current != last)
					{
						writer.print("<token POS=\'");
						String main_tag = current.get_tag_part(0);
						writer.print(main_tag + "\'");
						if(frame.high_tag_detail.isSelected())
						{
							//adjective or noun
							if(main_tag.equals("adjective") || main_tag.equals("noun"))
							{
								writer.print(" case=\'" + current.get_tag_part(1) + "\'");
								writer.print(" gender=\'" + current.get_tag_part(2) + "\'");
								writer.print(" number=\'" + current.get_tag_part(3) + "\'");
							}
							//other
							if(main_tag.equals("other"))
							{
								writer.print(" type= \'" + current.get_tag_part(1) + "\'");
							}
							//article
							if(main_tag.equals("article"))
							{
								writer.print(" function=\'" + current.get_tag_part(1) + "\'");
								writer.print(" case=\'" + current.get_tag_part(2) + "\'");
								writer.print(" gender=\'" + current.get_tag_part(3) + "\'");
								writer.print(" number=\'" + current.get_tag_part(4) + "\'");
							}
							//pronoun
							if(main_tag.equals("pronoun"))
							{
								//inflectionless
								if(current.get_tag_part(1).equals("inflectionless"))
								{
									writer.print(" mode=\'inflectionless\'");
								}
								else
								{
									//gender is only printed if a value has been set
									if(!current.get_tag_part(2).equals("--"))
									{
										writer.print(" gender=\'" + current.get_tag_part(2) + "\'");
									}
									writer.print(" number=\'" + current.get_tag_part(3) + "\'");
									writer.print(" case=\'" + current.get_tag_part(4) + "\'");
								}
							}
							//verb
							if(main_tag.equals("verb"))
							{
								if(current.get_tag_part(1).equals("--"))
								{
									writer.print(" voice=\'" + current.get_tag_part(2) + "\'");
									writer.print(" number=\'" + current.get_tag_part(3) + "\'");
									writer.print(" tense=\'" + current.get_tag_part(4) + "\'");
								}
								else if(current.get_tag_part(1).equals("infinitive"))
								{
									writer.print(" mode=\'infinitive\'");
									writer.print(" voice=\'" + current.get_tag_part(2) + "\'");
								}
								else
								{
									writer.print(" mode=\'participle\'");
								}
							}
						}
						writer.println(">");
						String token = current.get_token();
						token = token.replace("&", "&amp;");
						token = token.replace("<", "&lt;");
						token = token.replace(">", "&gt;");
						token = token.replace("\'", "&apos;");
						token = token.replace("\"", "&quot;");
						writer.println(token);
						
						writer.println("</token>");
						current = document.get_next_token(current);
					}
					//also print the last sentence token
					writer.print("<token POS=\'");
					String main_tag = current.get_tag_part(0);
					writer.print(main_tag + "\'");
					if(frame.high_tag_detail.isSelected())
					{
						//adjective or noun
						if(main_tag.equals("adjective") || main_tag.equals("noun"))
						{
							writer.print(" case=\'" + current.get_tag_part(1) + "\'");
							writer.print(" gender=\'" + current.get_tag_part(2) + "\'");
							writer.print(" number=\'" + current.get_tag_part(3) + "\'");
						}
						//other
						if(main_tag.equals("other"))
						{
							writer.print(" type= \'" + current.get_tag_part(1) + "\'");
						}
						//article
						if(main_tag.equals("article"))
						{
							writer.print(" function=\'" + current.get_tag_part(1) + "\'");
							writer.print(" case=\'" + current.get_tag_part(2) + "\'");
							writer.print(" gender=\'" + current.get_tag_part(3) + "\'");
							writer.print(" number=\'" + current.get_tag_part(4) + "\'");
						}
						//pronoun
						if(main_tag.equals("pronoun"))
						{
							//inflectionless
							if(current.get_tag_part(1).equals("inflectionless"))
							{
								writer.print(" mode=\'" + current.get_tag_part(1) + "\'");
							}
							else
							{
								writer.print(" gender=\'" + current.get_tag_part(2) + "\'");
								writer.print(" number=\'" + current.get_tag_part(3) + "\'");
								writer.print(" case=\'" + current.get_tag_part(4) + "\'");
							}
						}
						//verb
						if(main_tag.equals("verb"))
						{
							if(current.get_tag_part(1).equals("--"))
							{
								writer.print(" voice=\'" + current.get_tag_part(2) + "\'");
								writer.print(" number=\'" + current.get_tag_part(3) + "\'");
								writer.print(" tense=\'" + current.get_tag_part(4) + "\'");
							}
							else if(current.get_tag_part(1).equals("infinitive"))
							{
								writer.print(" mode=\'infinitive\'");
								writer.print(" voice=\'" + current.get_tag_part(2) + "\'");
							}
							else
							{
								writer.print(" mode=\'participle\'");
							}
						}
					}
					writer.println(">");
					String token = current.get_token();
					token = token.replace("&", "&amp;");
					token = token.replace("<", "&lt;");
					token = token.replace(">", "&gt;");
					token = token.replace("\'", "&apos;");
					token = token.replace("\"", "&quot;");
					writer.println(token);
					
					writer.println("</token>");
					
					writer.println("</sentence>");
					writer.flush();
					//go to the next sentence
					sentence = document.get_next_sentence(sentence);
				}
				writer.println("</document>");
				writer.flush();
			}
			writer.println("</folder>");
			writer.flush();
			writer.close();
		}
		catch(IOException e)
		{
			e.printStackTrace();
		}
	}
	
	//invoked when the exit is pressed
	protected void request_to_exit()
	{
		if(JOptionPane.showConfirmDialog(null,
						"Are you sure you want to quit?") == 0)
		{
			frame.dispose();
		}
	}
	
	//invoked when the tag detail changes
	protected void tag_detail_changed()
	{
		if(now_tagging)
		{
			JOptionPane.showMessageDialog(null,
					"Tagging operation in progress. Please wait.");
			return;
		}
		if(current_file_index == -1)
		{//no file is yet loaded
			return;
		}
		frame.panel.use_high_detail_tag(frame.high_tag_detail.isSelected(),
				current_token.get_tag_part(0),
				current_token.get_tag_part(1),
				current_token.get_tag_part(2),
				current_token.get_tag_part(3),
				current_token.get_tag_part(4));
		update_tag_panel();
	}
	
	//handles the request to change the format of the text
	protected void normalization_option_changed()
	{
		if(now_tagging)
		{
			JOptionPane.showMessageDialog(null,
					"Tagging operation in progress. Please wait.");
			return;
		}
		if(current_file_index == -1)
		{//no file is yet loaded
			return;
		}
		if(frame.normalize_text.isSelected())
		{
			frame.panel.text_area.setWrapStyleWord(true);
			frame.panel.text_area.setLineWrap(true);
		}
		else
		{
			frame.panel.text_area.setWrapStyleWord(true);
			frame.panel.text_area.setLineWrap(true);
		}
		for(int i=0; i<open_documents.length; ++i)
		{
			open_documents[i].normalize_document(frame.normalize_text.isSelected());
		}
		update_text();
	}
	
	//handles the tagging of the current document
	protected void request_to_tag()
	{
		if(current_file_index == -1)
		{//no file is yet loaded
			return;
		}
		if(now_tagging)
		{
			JOptionPane.showMessageDialog(null,
					"Tagging operation in progress. Please wait.");
			return;
		}
		now_tagging = true;
		//initiate the tagging handler
		tagging_handler = new Annotation_tool_tagging_handler();
		tagging_handler.set_parameters(
				(Integer)frame.k_spinner.getValue(),
				get_selected_difference_metric_code(),
				get_train_file_path(),
				frame.gain_ratio.getState(),
				frame.panel.progress_bar,
				new Token_document[]{open_documents[current_file_index]},
				this);
		tagging_handler.execute();
	}

	//handles the tagging of all the documents
	protected void request_to_tag_all()
	{
		if(current_file_index == -1)
		{//no file is yet loaded
			return;
		}
		if(now_tagging)
		{
			JOptionPane.showMessageDialog(null,
					"Tagging operation in progress. Please wait.");
			return;
		}
		now_tagging = true;
		//initiate the tagging handler
		tagging_handler = new Annotation_tool_tagging_handler();
		tagging_handler.set_parameters(
				(Integer)frame.k_spinner.getValue(),
				get_selected_difference_metric_code(),
				get_train_file_path(),
				frame.gain_ratio.getState(),
				frame.panel.progress_bar,
				open_documents,
				this);
		tagging_handler.execute();
	}
	
	//invoked when the tagging is completed
	protected void tagging_completed()
	{
		now_tagging = false;
		normalization_option_changed();
		update_highlights();
		frame.panel.use_high_detail_tag(frame.high_tag_detail.isSelected(),
				current_token.get_tag_part(0),
				current_token.get_tag_part(1),
				current_token.get_tag_part(2),
				current_token.get_tag_part(3),
				current_token.get_tag_part(4));
		update_tag_panel();
	}
	
	//updates the tag of the current token
	protected void tag_changed(String new_value, int box_number)
	{
		if(current_file_index == -1)
		{//no file is yet loaded
			return;
		}
		if(now_tagging)
		{
			JOptionPane.showMessageDialog(null,
					"Tagging operation in progress. Please wait.");
			return;
		}
		//if the main tag is changed then all secondary tags are nullified
		if(box_number == 0)
		{
			if(current_token.get_tag_part(0) != new_value)
			{
				current_token.set_tag_part(new_value, box_number);
				current_token.set_tag_part("--", 1);
				current_token.set_tag_part("--", 2);
				current_token.set_tag_part("--", 3);
				current_token.set_tag_part("--", 4);
			}
			//update the gui
			update_highlights();
		}
		else
		{
			//just update the secondary tag of the current token
			current_token.set_tag_part(new_value, box_number);
			//special occasions
			if(current_token.get_tag_part(0).equals("article") &&
					new_value.equals("indefinite"))
			{
				current_token.set_tag_part("singular", 4);
			}
			if(current_token.get_tag_part(0).equals("pronoun") &&
					new_value.equals("inflectionless"))
			{
				current_token.set_tag_part("--", 2);
				current_token.set_tag_part("--", 3);
				current_token.set_tag_part("--", 4);
			}
			if(current_token.get_tag_part(0).equals("verb"))
			{
				if(current_token.get_tag_part(1).equals("infinitive"))
				{
					current_token.set_tag_part("--", 3);
					current_token.set_tag_part("--", 4);
				}
				if(current_token.get_tag_part(1).equals("participle"))
				{
					current_token.set_tag_part("--", 2);
					current_token.set_tag_part("--", 3);
					current_token.set_tag_part("--", 4);
				}
			}
		}
		update_tag_panel();
		update_token_bar_message();
	}
	
	//finds the corresponding token for the given cursor position
	protected void cursor_changed(int index)
	{
		if(current_file_index == -1)
		{//no file is yet loaded
			return;
		}
		if(now_tagging)
		{
			JOptionPane.showMessageDialog(null,
					"Tagging operation in progress. Please wait.");
			return;
		}
		current_token =
			open_documents[current_file_index].get_token_in_cursor_position(index);
		update_tag_panel();
		update_token_bar_message();
	}
	
	//move to previous document
	protected void request_for_previous_document()
	{
		if(now_tagging)
		{
			JOptionPane.showMessageDialog(null,
					"Tagging operation in progress. Please wait.");
			return;
		}
		if(!(current_file_index == 0) && (current_file_index != -1))
		{
			current_file_index--;
			update_text();
			update_document_bar_message();
		}
	}
	
	//move to next document
	protected void request_for_next_document()
	{
		if(now_tagging)
		{
			JOptionPane.showMessageDialog(null,
					"Tagging operation in progress. Please wait.");
			return;
		}
		if(!(current_file_index == open_files.length-1) && (current_file_index != -1))
		{
			current_file_index++;
			update_text();
			update_document_bar_message();
		}
	}
	
	//move to previous token
	protected void request_for_previous_token()
	{
		if(now_tagging)
		{
			JOptionPane.showMessageDialog(null,
					"Tagging operation in progress. Please wait.");
			return;
		}
		current_token = open_documents[current_file_index].
			get_previous_token(current_token);
		update_tag_panel();
		update_highlights();
		update_token_bar_message();
	}
	
	//move to next token
	protected void request_for_next_token()
	{
		if(now_tagging)
		{
			JOptionPane.showMessageDialog(null,
					"Tagging operation in progress. Please wait.");
			return;
		}
		current_token = open_documents[current_file_index].
			get_next_token(current_token);
		update_tag_panel();
		update_highlights();
		update_token_bar_message();
	}
	
	//updates the text of the panel
	private void update_text()
	{
		if(frame.normalize_text.isSelected())
		{
			frame.panel.text_area.setText(open_documents[current_file_index].get_normalized_text());
		}
		else
		{
			frame.panel.text_area.setText(open_files[current_file_index][1]);
		}
	}
	
	//updates the message of the document_bar in the panel
	private void update_document_bar_message()
	{
		frame.panel.document_bar.setText("Current Document: " + open_files[current_file_index][0] +
				" - No: " + (current_file_index+1) +
				" of " + open_files.length);
	}
	
	//updates the message of the token_bar in the panel
	private void update_token_bar_message()
	{
		frame.panel.token_bar.setText("Current Token: " + current_token.get_token());
	}
	
	//updates the tag panel in the main panel so that it shows the tag of the
	//current token
	private void update_tag_panel()
	{
		frame.panel.show_combo_boxes_for_values(
				current_token.get_tag_part(0),
				current_token.get_tag_part(1),
				current_token.get_tag_part(2),
				current_token.get_tag_part(3),
				current_token.get_tag_part(4));
	}

	//update the highlights of the current document
	private void update_highlights()
	{
		//get all highlights of the document
		DefaultHighlighter.Highlight[] highlights =
			frame.panel.text_area.getHighlighter().getHighlights();
		//and remove them
		for(int i=0; i<highlights.length; ++i)
		{
			frame.panel.text_area.getHighlighter().removeHighlight(highlights[i]);
		}
		Iterator<Token> token_iterator =
			open_documents[current_file_index].get_tokens().iterator();
		Token token = null;
		//for each token in the document
		while(token_iterator.hasNext())
		{
			token = token_iterator.next();
			//get the color for the main tag of the token
			DefaultHighlighter.DefaultHighlightPainter highlighter =
					new DefaultHighlighter.DefaultHighlightPainter(
							tag_to_color.get(token.get_tag_part(0)));
			try
			{
				//special highlight for the current document
				if(token == current_token)
				{
					frame.panel.text_area.getHighlighter().addHighlight(
							token.get_starting_position(),
							token.get_ending_position(),
							new DefaultHighlighter.DefaultHighlightPainter(
									new Color(255, 255, 100)));
					continue;
				}
				//highlight the current token accordingly
				frame.panel.text_area.getHighlighter().addHighlight(
					token.get_starting_position(),
					token.get_ending_position(), highlighter);
			}
			catch(BadLocationException e)
			{
					e.printStackTrace();
			}
		}
	}
	
	//return the code of the selected difference metric
	private int get_selected_difference_metric_code()
	{
		if(frame.modified_value.isSelected())
		{
			return 1;
		}
		return 0;
	}
	
	//returns the path of the selected train file
	private File get_train_file_path()
	{
		if(frame.default_train_set.isSelected())
		{
			if(frame.high_tag_detail.isSelected())
			{
				return new File(default_train_file_high_detail);
			}
			else
			{
				return new File(default_train_file_low_detail);
			}
		}
		else
		{
			return frame.custom_train_file;	
		}
	}
	
	private Annotation_tool_frame frame;
	private String[][] open_files;
	private Token_document[] open_documents;
	private TreeMap<String, Color> tag_to_color;
	private int current_file_index;
	private Token current_token;
	private Annotation_tool_tagging_handler tagging_handler;
	private boolean now_tagging;
	private String default_train_file_high_detail;
	private String default_train_file_low_detail;
	public static String file_separator;
	
}
