package gui.annotation_tool;

import instance_provider.*;
import javax.swing.*;

import converter.Converter;

import java.io.*;
import document.Token_document;
import classifier.*;
import java.util.*;
import document.*;

public class Annotation_tool_tagging_handler extends SwingWorker<Void, Void>
{
	
	public void set_parameters(int k_in, int difference_metric_code_in,
			File train_file_in, boolean gain_ratio_in, JProgressBar progress_bar_in,
			Token_document[] documents_in, Annotation_tool_gui_manager manager_in)
	{
		k = k_in;
		difference_metric_code = difference_metric_code_in;
		train_file = train_file_in;
		gain_ratio = gain_ratio_in;
		progress_bar = progress_bar_in;
		documents = documents_in;
		manager = manager_in;
	}
	
	public Void doInBackground()
	{
	    Dummy_Loader loader = new Dummy_Loader(progress_bar);
	    loader.execute();
	    
	    //create the classifier
	    Knn knn = new Knn(k);
	    try
	    {
	    	knn.train(get_train_instances(), difference_metric_code);
	    }
	    catch(Exception e)
	    {
	    	loader.stop();
	    }
	    knn.use_properties_weight(gain_ratio);

	  	loader.stop();
	  	
		//tag all given documents
	    for(int i=0; i<documents.length; ++i)
	    {
	    	tag_document(i, knn);
	    }
		return null;
	}
	
	//invoked automatically when the doInBackground method finishes
	public void done()
	{
	    progress_bar.setString("Tagging Completed");
	    manager.tagging_completed();
	}
	
	//extracts all the train instances
	private ArrayList<Instance> get_train_instances()
	{
		Instance_provider provider = new Train_file_instance_provider();
		provider.set_number_of_instances_to_provide(MAX_NUM);
		provider.parse_file(train_file.getPath());
		return provider.get_instances();
	}
	
	//updates the tags
	private void tag_document(int doc_index, Knn knn)
	{
		progress_bar.setString("Tagging Document No. "
				+ (doc_index+1) + " of " + documents.length);
		progress_bar.setValue(0);
		int current_token_index = 0;
		int total_tokens = documents[doc_index].get_tokens().size();
		progress_bar.setMaximum(total_tokens);
		//for each token in the document
		Iterator<Token> tokens_it = documents[doc_index].get_tokens().iterator();
		while(tokens_it.hasNext())
		{
			Token temp = tokens_it.next();
			//create the corresponding instance and classify it
			String category = knn.classify(get_instance_from_token(temp, doc_index, knn));
			//and update its tag
			update_token_tag(temp, category);
			//update the progress bar
			current_token_index++;
			progress_bar.setValue(current_token_index);
		}
		
	}
	
	//creates and returns the corresponding instance of a given token
	private Instance get_instance_from_token(Token token, int doc_index, Knn knn)
	{
		Instance instance = new Instance();
		//word ending
		instance.set_property_value(0, get_word_ending(token.get_token()));
		//the length of the word
		instance.set_property_value(1, (new Integer(token.get_token().length())).toString());
		//if the word has apostrophe
		if(token.get_token().contains("\'"))
		{
			instance.set_property_value(2, "1");
		}
		else
		{
			instance.set_property_value(2, "0");
		}
		//if the word has digit
		instance.set_property_value(3, "0");
		//for each character of the word
		for(int i=0; i<token.get_token().length(); ++i)
		{//if it is a digit
			if(Character.isDigit(token.get_token().charAt(i)))
			{
				instance.set_property_value(3, "1");
			}
		}
		//if the word has dot
		if(token.get_token().contains("."))
		{
			instance.set_property_value(4, "1");
		}
		else
		{
			instance.set_property_value(4, "0");
		}
		//if the word has comma
		if(token.get_token().contains(","))
		{
			instance.set_property_value(5, "1");
		}
		else
		{
			instance.set_property_value(5, "0");
		}
		//if the word has latin character(that is A-Z or a-z)
		instance.set_property_value(6, "0");
		//for each character of the word
		for(int i=0; i<token.get_token().length(); ++i)
		{//if it is a latin character
			int num_code = (int)token.get_token().charAt(i);
			if((num_code>=65 && num_code<=90) || (num_code>=97 && num_code<=122))
			{
				instance.set_property_value(6, "1");
			}
		}
		//the ambitag of the word
		instance.set_property_value(7, knn.get_ambitag(token.get_token()));
		//the ending of the next word
		Token next = documents[doc_index].get_next_token(token);
		//if this token is the last one
		if(token == next)
		{
			instance.set_property_value(8, "null");
		}
		else
		{
			instance.set_property_value(8, get_word_ending(next.get_token()));
		}
		//the ambitag of the next word
		//if this word is the last one
		if(token == null)
		{
			instance.set_property_value(9, "null");
		}
		else
		{
			instance.set_property_value(9, knn.get_ambitag(next.get_token()));
		}
		//the word before the previous word
		Token previous = documents[doc_index].get_previous_token(token);
		Token pre_previous = documents[doc_index].get_previous_token(previous);
		//if there is no word before the previous word
		if(previous == token || pre_previous == previous)
		{
			instance.set_property_value(10, "null");
		}
		else
		{
			instance.set_property_value(10, pre_previous.get_token());
		}
		//the ending of the word before the previous word
		//if there is no word before the previous word
		if(previous == token || pre_previous == previous)
		{
			instance.set_property_value(11, "null");
		}
		else
		{
			instance.set_property_value(11, get_word_ending(pre_previous.get_token()));
		}
		//the ending of the previous word
		//if there is no previous word
		if(previous == token)
		{
			instance.set_property_value(12, "null");
		}
		else
		{
			instance.set_property_value(12, get_word_ending(previous.get_token()));
		}
		//the previous word
		//if there is no previous word
		if(previous == token)
		{
			instance.set_property_value(13, "null");
		}
		else
		{
			instance.set_property_value(13, previous.get_token());
		}
		//the ambitag of the previous word
		//if there is no previous word
		if(previous == token)
		{
			instance.set_property_value(14, "null");
		}
		else
		{
			instance.set_property_value(14, knn.get_ambitag(previous.get_token()));
		}
		//the category
		instance.set_property_value(15, "?");
		//the instance word
		instance.set_property_value(16, token.get_token());
		//the next word
		if(token == next)
		{
			instance.set_property_value(17, "null");
		}
		else
		{
			instance.set_property_value(17, next.get_token());
		}
		return instance;
	}
	
	//returns the ending of a word
	private String get_word_ending(String word)
	{
		//if the word is less than 3 characters long
		if(word.length()<=3)
		{//its ending is the word itself
			return word;
		}
		//else the ending is the three last characters
		return word.substring(word.length()-3, word.length());
	}
	
	//updates the tag of the given token according to the given category
	private void update_token_tag(Token token, String category)
	{
		token.set_tag_part(category.split("/")[0], 0);
		token.set_tag_part(category.split("/")[1], 1);
		token.set_tag_part(category.split("/")[2], 2);
		token.set_tag_part(category.split("/")[3], 3);
		token.set_tag_part(category.split("/")[4], 4);
	}
	
	private int k;
	private int difference_metric_code;
	private File train_file;
	private boolean gain_ratio;
	private JProgressBar progress_bar;
	private Token_document[] documents;
	private Annotation_tool_gui_manager manager;
	private HashMap<String, String> category_to_tag;
	private final int MAX_NUM = 50000;//*******************************************************
	
	private class Dummy_Loader extends SwingWorker<Void, Void>
	{
		
		public Dummy_Loader(JProgressBar b)
		{
			bar = b;
			change_message = true;
			message = "Creating Classifier   ";
		}
		
		public void stop()
		{
			change_message = false;
		}
		
		public Void doInBackground()
		{
		    progress_bar.setStringPainted(true);
		    progress_bar.setString(message);
		    try
		    {
		    	Thread.sleep(1000);
		    }
		    catch(InterruptedException e)
		    {
		    	e.printStackTrace();
		    }
		    while(change_message)
		    {
		    	try
		    	{
		    		update_message();
		    		progress_bar.setString(message);
		    		Thread.sleep(1000);
		    	}
		    	catch(InterruptedException e)
		    	{
		    		e.printStackTrace();
		    	}
		    }
		    return null;
		}
		
		private void update_message()
		{
			if(message.equals("Creating Classifier   "))
			{
				message = "Creating Classifier.  ";
			}
			else if(message.equals("Creating Classifier.  "))
			{
				message = "Creating Classifier.. ";
			}
			else if(message.equals("Creating Classifier.. "))
			{
				message = "Creating Classifier...";
			}
			else
			{
				message = "Creating Classifier   ";
			}
		}
		
		private JProgressBar bar;
		private boolean change_message;
		private String message;
		
	}
	
}