package instance_provider;

import java.io.*;
import java.util.*;
import classifier.*;

public class Non_annotation_instance_provider implements Instance_provider
{

	public Non_annotation_instance_provider(Knn knn)
	{
		this.knn = knn;
	}
	
	public void parse_file(String path)
	{
		file_path = path;
	}
	
	public void set_number_of_instances_to_provide(int i)
	{
		number_of_instances = i;
	}
	
	public ArrayList<Instance> get_instances()
	{
		if(file_path == null)
		{
			throw new IllegalArgumentException("Path has not yet been set.");
		}
		int counter = 0;
		//here we hold all the words
		//we hold as many words as the number_of_instances implies
		String[] words = new String[number_of_instances];
		//here we temporarily hold the words of a line
		String[] temp = null;
		BufferedReader reader = null;
		try
		{
			reader = new BufferedReader(new InputStreamReader(
					new FileInputStream(file_path), "UTF-8"));
			String line = reader.readLine();
			//for each line of the given text
			while(line != null)
			{
				temp = line.trim().split(" ");
				//for each word in it
				for(int i=0; i<temp.length; ++i)
				{
					words[counter] = temp[i];
					counter++;
					if(counter == number_of_instances)
					{
						break;
					}
				}
				if(counter == number_of_instances)
				{
					break;
				}
				line = reader.readLine();
			}
		}
		catch(FileNotFoundException e)
		{
			System.out.println("Incorrect or missing file.");
			e.printStackTrace();
		}
		catch(IOException e)
		{
			System.out.println("Problem occured while reading file " + file_path + ".");
			e.printStackTrace();
		}
		finally
		{
			try{reader.close();}catch(Exception e){e.printStackTrace();}
		}
		return proccess_words(words);
	}
	
	//for each word we found in the parsing phase we now construct a different instance
	private ArrayList<Instance> proccess_words(String[] words)
	{
		ArrayList<Instance> instances = new ArrayList<Instance>();
		Instance temp = null;
		//for each word
		for(int i=0; i<words.length; ++i)
		{
			temp = new Instance();
			//complete the property values of the new instance
			for(int j=0; j<=temp.get_number_of_properties()+2; j++)
			{
				temp.set_property_value(j, extract_property_value(words, i, j));
			}
			instances.add(temp);
		}
		return instances;
	}
	
	private String extract_property_value(String[] words, int word_index, int property_index)
	{
		switch(property_index)
		{
			//word ending
			case 0:
			{
				//if the word is less than 3 characters long
				if(words[word_index].length()<=3)
				{//its ending is the word itself
					return words[word_index];
				}
				//else return the three last characters
				return words[word_index].substring(
						words[word_index].length()-3,
						words[word_index].length());
			}
			//the length of the word
			case 1:
			{
				return (new Integer(words[word_index].length())).toString();
			}
			//if the word has apostrophe
			case 2:
			{
				if(words[word_index].contains("\'"))
				{
					return "1";
				}
				return "0";
			}
			//if the word has digit
			case 3:
			{
				//for each character of the word
				for(int i=0; i<words[word_index].length(); ++i)
				{//if it is a digit
					if(Character.isDigit(words[word_index].charAt(i)))
					{
						return "1";
					}
				}
				return "0";
			}
			//if the word has dot
			case 4:
			{
				if(words[word_index].contains("."))
				{
					return "1";
				}
				return "0";
			}
			//if the word has comma
			case 5:
			{
				if(words[word_index].contains(","))
				{
					return "1";
				}
				return "0";
			}
			//if the word has latin character(that is A-Z or a-z)
			case 6:
			{
				//for each character of the word
				for(int i=0; i<words[word_index].length(); ++i)
				{//if it is a latin character
					int num_code = (int)words[word_index].charAt(i);
					if((num_code>=65 && num_code<=90) || (num_code>=97 && num_code<=122))
					{
						return "1";
					}
				}
				return "0";
			}
			//the ambitag of the word
			case 7:
			{
				return knn.get_ambitag(words[word_index]);
			}
			//the ending of the next word
			case 8:
			{
				//if this word is the last one
				if(words.length == word_index+1)
				{//return null
					return "null";
				}
				//else return the ending of the next word
				return extract_property_value(words, word_index+1, 0);
			}
			//the ambitag of the next word
			case 9:
			{
				//if this word is the last one
				if(words.length == word_index+1)
				{//return null
					return "null";
				}
				//else return the ambitag of the next word
				return extract_property_value(words, word_index+1, 7);
			}
			//the word before the previous word
			case 10:
			{
				//if this word is one of the first two words
				if(word_index < 2)
				{//return null
					return "null";
				}
				//else return the word before the previous word
				return words[word_index-2];
			}
			//the ending of the word before the previous word
			case 11:
			{
				//if this word is one of the first two words
				if(word_index < 2)
				{//return null
					return "null";
				}
				//else return the ending of the word before the previous word
				return extract_property_value(words, word_index-2, 0);
			}
			//the ending of the previous word
			case 12:
			{
				//if this word is the first word
				if(word_index < 1)
				{//return null
					return "null";
				}
				//else return the ending of the previous word
				return extract_property_value(words, word_index-1, 0);
			}
			//the previous word
			case 13:
			{
				//if this word is the first word
				if(word_index < 1)
				{//return null
					return "null";
				}
				//else return the previous word
				return words[word_index-1];
			}
			//the ambitag of the previous word
			case 14:
			{
				//if this word is the first word
				if(word_index < 1)
				{//return null
					return "null";
				}
				//else return the ambitag of the previous word
				return extract_property_value(words, word_index-1, 7);
			}
			//the category
			case 15:
			{
				return "?";
			}
			//the word itself
			case 16:
			{
				return words[word_index];
			}
			//the next word
			case 17:
			{
				if(word_index < words.length-1)
				{
					return words[word_index+1];
				}
				return "null";
			}
			default:return "null";
		}
	}
	
	private String file_path = null;
	private int number_of_instances;
	private Knn knn;
	
}
