package instance_provider;

import java.util.*;

import classifier.*;

import java.io.*;
import javax.swing.JOptionPane;
import converter.*;

public class Train_file_instance_provider implements Instance_provider
{

	//sets the file where the instances will be found
	public void parse_file(String file)
	{
		file_path = file;
	}
	
	//sets the number on instances that will be constructed at most
	public void set_number_of_instances_to_provide(int num)
	{
		number_of_instances = num;
	}
	
	//extracts the instances and returns an array list of them
	public ArrayList<Instance> get_instances()
	{
		if(file_path == null)
		{
			throw new IllegalArgumentException("Path has not yet been set.");
		}
		File file = new File(file_path);
		//get the train files
		File[] train_files = new File[0];
		if(file.isDirectory())
		{
			train_files = file.listFiles();
		}
		else
		{
			train_files = new File[]{file};
		}
		//check if the train files have the correct format
		for(int i=0; i<train_files.length; ++i)
		{
			if(!train_files[i].getName().substring(
					train_files[i].getName().length()-5).equalsIgnoreCase("train"))
			{
				JOptionPane.showMessageDialog(null, "Classifier can only be trained in"+
						" files of type .train.");
				throw new IllegalArgumentException("Classifier can only be trained in"+
						" files of type .train.");
			}
		}
		//keep in an array the train paradigms
		String[] train_paradeigms = new String[number_of_instances];
		int counter = 0;
		//get the train paradigms
		for(int i=0; i<train_files.length && counter < number_of_instances; ++i)
		{
			try
			{
				BufferedReader reader = new BufferedReader(new InputStreamReader(
						new FileInputStream(train_files[i]), "UTF-8"));
				String line = reader.readLine();
			    //if first character is not a text character remove it
			    if((int)line.charAt(0) == 65279)
			    {
			    	line = line.substring(1, line.length());
			    }
				while(line != null && counter < number_of_instances)
				{
					train_paradeigms[counter] = line;
					counter++;
					line = reader.readLine();
				}
			}
			catch(FileNotFoundException e)
			{
				e.printStackTrace();
			}
			catch(IOException e)
			{
				e.printStackTrace();
			}
		}
		if(counter < number_of_instances)
		{
			String[] temp = new String[counter];
			for(int i=0; i<counter; ++i)
			{
				temp[i] = train_paradeigms[i];
			}
			train_paradeigms = temp;
		}
		return get_instances_from_paradeigms(train_paradeigms);
	}
	
	//extracts the instances from an array with lines from the train file
	private ArrayList<Instance> get_instances_from_paradeigms(String[] train_paradeigms)
	{
		//encode categories to save memory
		for(int i=0; i<train_paradeigms.length; ++i)
		{
			train_paradeigms[i] = train_paradeigms[i].split(" ")[0] + " " +
			train_paradeigms[i].split(" ")[1] + " " + train_paradeigms[i].split(" ")[2] + " " +
			Converter.reverse(train_paradeigms[i].split(" ")[3]) + " " + train_paradeigms[i].split(" ")[4];
		}
		//we can calculate the ambitags of all the words from a train file
		//this is all the tags found for the given word separated by a hyphen
		//sorted by the times each tag appears for the given word
		//the same is done for the ending of each word
		word_tag_times =
			new HashMap<String, TreeMap<String, Integer>>();
		ending_tag_times =
			new HashMap<String, TreeMap<String, Integer>>();
		for(int i=0; i<train_paradeigms.length; ++i)
		{
			String word = train_paradeigms[i].split(" ")[2];
			//if the word has been found before
			if(word_tag_times.containsKey(word))
			{
				//if the current tag has been found before for the given word
				if(word_tag_times.get(word).containsKey(
						train_paradeigms[i].split(" ")[3]))
				{
					//add one to the number of appearances of this tag for this word
					word_tag_times.get(word).put(
							train_paradeigms[i].split(" ")[3],
							new Integer(word_tag_times.get(word).get(
									train_paradeigms[i].split(" ")[3]).intValue()+1));
				}
				//else add the new tag for the word
				else
				{
					word_tag_times.get(word).put(
							train_paradeigms[i].split(" ")[3],
							new Integer(1));
				}
			}
			//else insert the new word with its only tag
			else
			{
				TreeMap<String, Integer> temp = new TreeMap<String, Integer>();
				temp.put(train_paradeigms[i].split(" ")[3], new Integer(1));
				word_tag_times.put(word, temp);
			}
			String word_ending = "";
			
			//if the word is less than 3 characters long
			if(word.length()<=3)
			{//its ending is the word itself
				word_ending = word;
			}
			//else keep the three last characters
			else
			{
				word_ending = word.substring(word.length()-3, word.length());
			}
			
			//if the ending of the word has been found before
			if(ending_tag_times.containsKey(word_ending))
			{
				//if the current tag has been found before for the given ending
				if(ending_tag_times.get(word_ending).containsKey(
						train_paradeigms[i].split(" ")[3]))
				{
					//add one to the number of appearances of this tag for this ending
					ending_tag_times.get(word_ending).put(
							train_paradeigms[i].split(" ")[3],
							new Integer(ending_tag_times.get(word_ending).get(
									train_paradeigms[i].split(" ")[3]).intValue()+1));
				}
				//else add the new tag for the ending
				else
				{
					ending_tag_times.get(word_ending).put(
							train_paradeigms[i].split(" ")[3],
							new Integer(1));
				}
			}
			//else insert the new word ending with its only tag
			else
			{
				TreeMap<String, Integer> temp = new TreeMap<String, Integer>();
				temp.put(train_paradeigms[i].split(" ")[3], new Integer(1));
				ending_tag_times.put(word_ending, temp);
			}
		}
		ArrayList<Instance> instances = new ArrayList<Instance>();
		Instance temp = null;

		//for each train paradigm
		for(int i=0; i<train_paradeigms.length; ++i)
		{
			temp = new Instance();
			//complete the property values of the new instance
			for(int j=0; j<temp.get_number_of_properties()+3; j++)
			{
				temp.set_property_value(j, extract_property_value(train_paradeigms,
						i, j, word_tag_times, ending_tag_times));
			}
			instances.add(temp);
		}
		return instances;
	}
	
	private String extract_property_value(String[] train_paradeigms, int paradeigm_index,
			int property_index, HashMap<String, TreeMap<String, Integer>> word_tag_times,
			HashMap<String, TreeMap<String, Integer>> ending_tag_times)
	{
		String word = train_paradeigms[paradeigm_index].split(" ")[2];
		switch(property_index)
		{
			//word ending
			case 0:
			{
				return get_ending(word);
			}
			//the length of the word
			case 1:
			{
				if(word.equals("null"))
				{
					return "0";
				}
				return (new Integer(word.length())).toString();
			}
			//if the word has apostrophe
			case 2:
			{
				if(word.contains("\'"))
				{
					return "1";
				}
				return "0";
			}
			//if the word has digit
			case 3:
			{
				//for each character of the word
				for(int i=0; i<word.length(); ++i)
				{//if it is a digit
					if(Character.isDigit(word.charAt(i)))
					{
						return "1";
					}
				}
				return "0";
			}
			//if the word has dot
			case 4:
			{
				if(word.contains("."))
				{
					return "1";
				}
				return "0";
			}
			//if the word has comma
			case 5:
			{
				if(word.contains(","))
				{
					return "1";
				}
				return "0";
			}
			//if the word has latin character(that is A-Z or a-z)
			case 6:
			{
				if(word.equals("null"))
				{
					return "0";
				}
				//for each character of the word
				for(int i=0; i<word.length(); ++i)
				{//if it is a latin character
					int num_code = (int)word.charAt(i);
					if((num_code>=65 && num_code<=90) || (num_code>=97 && num_code<=122))
					{
						return "1";
					}
				}
				return "0";
			}
			//the ambitag of the word
			case 7:
			{
				return get_ambitag(word);
			}
			//the ending of the next word
			case 8:
			{
				if(train_paradeigms[paradeigm_index].split(" ")[4].equals("null"))
				{
					return "null";
				}
				//if the next word is less than 3 characters long
				if(train_paradeigms[paradeigm_index].split(" ")[4].length()<=3)
				{//its ending is the word itself
					return train_paradeigms[paradeigm_index].split(" ")[4];
				}
				//else return the three last characters
				return train_paradeigms[paradeigm_index].split(" ")[4].substring(
						train_paradeigms[paradeigm_index].split(" ")[4].length()-3,
						train_paradeigms[paradeigm_index].split(" ")[4].length());
			}
			//the ambitag of the next word
			case 9:
			{
				return get_ambitag(train_paradeigms[paradeigm_index].split(" ")[4]);
			}
			//the word before the previous word
			case 10:
			{
				return train_paradeigms[paradeigm_index].split(" ")[0];
			}
			//the ending of the word before the previous word
			case 11:
			{
				if(train_paradeigms[paradeigm_index].split(" ")[0].equals("null"))
				{
					return "null";
				}
				//if the word is less than 3 characters long
				if(train_paradeigms[paradeigm_index].split(" ")[0].length()<=3)
				{//its ending is the word itself
					return train_paradeigms[paradeigm_index].split(" ")[0];
				}
				//else return the three last characters
				return train_paradeigms[paradeigm_index].split(" ")[0].substring(
						train_paradeigms[paradeigm_index].split(" ")[0].length()-3,
						train_paradeigms[paradeigm_index].split(" ")[0].length());
			}
			//the ending of the previous word
			case 12:
			{
				if(train_paradeigms[paradeigm_index].split(" ")[1].equals("null"))
				{
					return "null";
				}
				//if the word is less than 3 characters long
				if(train_paradeigms[paradeigm_index].split(" ")[1].length()<=3)
				{//its ending is the word itself
					return train_paradeigms[paradeigm_index].split(" ")[1];
				}
				//else return the three last characters
				return train_paradeigms[paradeigm_index].split(" ")[1].substring(
						train_paradeigms[paradeigm_index].split(" ")[1].length()-3,
						train_paradeigms[paradeigm_index].split(" ")[1].length());
			}
			//the previous word
			case 13:
			{
				return train_paradeigms[paradeigm_index].split(" ")[1];
			}
			//the ambitag of the previous word
			case 14:
			{
				return get_ambitag(train_paradeigms[paradeigm_index].split(" ")[1]);
			}
			//the category
			case 15:
			{
				return train_paradeigms[paradeigm_index].split(" ")[3];
			}
			//the word itself
			case 16:
			{
				return train_paradeigms[paradeigm_index].split(" ")[2];
			}
			//the next word
			case 17:
			{
				return train_paradeigms[paradeigm_index].split(" ")[4];
			}
			default:return "null";
		}
	}
	
	//returns the ending of a word
	private String get_ending(String word)
	{
		if(word.equals("null"))
		{
			return "null";
		}
		//if the word is less than 3 characters long
		if(word.length()<=3)
		{//its ending is the word itself
			return word;
		}
		//else return the three last characters
		return word.substring(
				word.length()-3,
				word.length());
	}
	
	//returns the ambitag of the word
	private String get_ambitag(String word)
	{
		TreeMap<String, Integer> tag_times = word_tag_times.get(word);
		if(tag_times == null)
		{
			tag_times = ending_tag_times.get(get_ending(word));
		}
		if(tag_times == null)
		{
			return "unknown";
		}
		StringBuffer ambitag = new StringBuffer("");
		TreeMap<Integer, String> times_tag = new TreeMap<Integer, String>();
		Iterator<String> it = tag_times.keySet().iterator();
		while(it.hasNext())
		{
			String tag = it.next();
			Integer times = tag_times.get(tag);
			if(times_tag.containsKey(times))
			{
				times_tag.put(times, times_tag.get(times) + "-" + tag);
			}
			else
			{
				times_tag.put(times, tag);
			}
		}
		//a tag comes first in the ambitag if it appears more times than another
		//if more than one tags appear the same number of times they are sorted
		//according to their string representation
		while(!times_tag.isEmpty())
		{
			ambitag.append(times_tag.pollLastEntry().getValue() + "-");
		}
		return ambitag.substring(0, ambitag.length()-1);
	}
	
	private int number_of_instances;
	private String file_path;
	private HashMap<String, TreeMap<String, Integer>> word_tag_times;
	private HashMap<String, TreeMap<String, Integer>> ending_tag_times;
	
}
